/*
 * Created on 21.04.2004
 *
 * (C) Peter Weigerber <peter.weissgerber@ku-eichstaett.de>
 * and Michael Burch <michael.burch@ku-eichstaett.de>
 * 
 * For license information see the LICENSE file that is shipped
 * with the source code.
 * 
 * 
 */
package org.eposoft.epos.eposee.pixmap.gui;

import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.SWT;

/** This class presents a canvas that contains a color scale.
 * 
 * @author Peter Weigerber <peter.weissgerber@ku-eichstaett.de>
 * @author Michael Burch <michael.burch@ku-eichstaett.de>
 *
 */
public class ColorScaleCanvas extends Canvas {

	private int min = 0;
	private int max = 200;
	private int table = 0;
	private RGB[] scale;
	private String name;
	private final int upper = 15;
	private final int medium = 20;
	private final int lower = 25;
	private boolean activated;

	/** Create a new color scale canvas.
	 * 
	 * @param arg0 composite to which the canvas belongs
	 * @param arg1 SWT flags, @see org.eclipse.swt.SWT
	 * @param colorscale colors building the color scale
	 * @param scalename name of the color scale
	 */
	public ColorScaleCanvas(
		Composite arg0,
		int arg1,
		RGB[] colorscale,
		String scalename) {
		super(arg0, arg1);
		scale = colorscale;
		name = scalename;
		this.addMouseListener(new MyMouseListener());
		this.addPaintListener(new MyPaintListener());
		activated = false;
	}

	public class MyMouseListener implements MouseListener {

		/*
		 *  (non-Javadoc)
		 * @see org.eclipse.swt.events.MouseListener#mouseDoubleClick(org.eclipse.swt.events.MouseEvent)
		 */
		public void mouseDoubleClick(MouseEvent arg0) {
		}

		/*
		 *  (non-Javadoc)
		 * @see org.eclipse.swt.events.MouseListener#mouseDown(org.eclipse.swt.events.MouseEvent)
		 */
		public void mouseDown(MouseEvent e) {
			// only react on mouse event if activated
			if (activated) {
				int xpos = e.x;
				int ypos = e.y;
				if ((ypos > (upper - 5)) && (ypos < medium)) {
					max = xpos - 5;
					if (max > 200) {
						max = 200;
					}
					if (max <= min) {
						max = min + 1;
					}
					redraw();
				}
				else if ((ypos > medium) && (ypos < (lower + 5))) {
					min = xpos - 5;
					if (min < 0) {
						min = 0;
					}
					if (min >= max) {
						min = max - 1;
					}
					redraw();
				}
			}
		}

		/*
		 *  (non-Javadoc)
		 * @see org.eclipse.swt.events.MouseListener#mouseUp(org.eclipse.swt.events.MouseEvent)
		 */
		public void mouseUp(MouseEvent arg0) {
		}

	}

	public class MyPaintListener implements PaintListener {

		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.PaintListener#paintControl(org.eclipse.swt.events.PaintEvent)
		 */
		public void paintControl(PaintEvent arg0) {
			GC g = arg0.gc;

			for (int k = 0; k <= 200; k++) {
				g.setForeground(
					new Color(
						getDisplay(),
						scale[(int) ((scale.length - 1) * 0.005 * k)]));
				g.drawLine(5 + k, upper, 5 + k, lower);
			}
			g.setForeground(new Color(getDisplay(), 0, 0, 0));
			//g.drawLine(5,25,205,25);
			if (name != null) {
				g.setFont(
					new Font(
						getDisplay(),
						"SansSerif",
						7,
						SWT.ITALIC | SWT.BOLD));
				g.drawString(name, 245, lower - 10);
			}

			g.setForeground(new Color(getDisplay(), 0, 0, 0));
			g.setFont(
				new Font(getDisplay(), "SansSerif", 6, SWT.ITALIC | SWT.BOLD));
			g.drawRectangle(4, 20 + 30, 201, 10);

			// draw max/min positions if activated
			if (activated) {
				g.setBackground(new Color(getDisplay(), 0, 0, 0));
				int[] points2 = { 1 + max, upper - 1 + 30 * table,
					9 + max, upper - 1 + 30 * table,
					5 + max, medium + 30 * table };
				g.fillPolygon(points2);
				g.setBackground(new Color(getDisplay(), 255, 255, 255));
				g.drawString("max", max + 10, upper - 1 + 30 * table - 9);
				g.setBackground(new Color(getDisplay(), 0, 0, 0));
				int[] points3 = { 1 + min, lower + 2 + 30 * table,
					9 + min, lower + 2 + 30 * table,
					5 + min, medium + 25 * table };
				g.fillPolygon(points3);
				g.setBackground(new Color(getDisplay(), 255, 255, 255));
				g.drawString("min", 10 + min, lower + 13 + 30 * table - 9);
			}
		}
	}

	/** Activate or deactivate this color scale.
	 * 
	 * @param a_activated true for activate, false for deactivate
	 */
	public void setActivated(boolean a_activated) {
		activated = a_activated;
		redraw();
	}

	/** Is this color scale activated?
	 * 
	 * @return boolean anwer
	 */
	public boolean isActivated() {
		return activated;
	}

	/** Get minimum selection.
	 * 
	 * @return int
	 */
	public int getMin() {
		return this.min;
	}
	
	/** Get maximum selection.
	 * 
	 * @return int
	 */
	public int getMax() {
		return this.max;
	}
	
	/** Get the selected color scale.
	 * 
	 * @return array of RGB values
	 */
	public RGB[] getSelectedColors() {
		RGB[] selected = new RGB[max - min];
		for (int i = min; i < max; i++) {
			selected[i - min] = scale[(int) ((scale.length - 1) * 0.005 * i)];
		}
		return selected;
	}

}
