/*
 * Created on 16.04.2004
 * 
 * (C) Peter Weigerber <peter.weissgerber@ku-eichstaett.de>
 * 
 * For license information see the LICENSE file that is shipped
 * with the source code.
 * 
 * 
 */
package org.eposoft.epos.eposee.pixmap.gui;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eposoft.epos.eposee.pixmap.data.SparePixMap;


/** Main class of the pixelmap viewer.
 * 
 * @author Peter Weigerber <peter.weissgerber@ku-eichstaett.de>
 * 
 */
public class EPOSpix {

	public final static String x_bar_empty_text =
		"No item selected on the x-axis.";
	public final static String y_bar_empty_text =
		"No item selected on the y-axis.";
	public final static String coupling_bar_empty_text =
		"No coupling selected.";

	private Shell mainframe;
	private PixMapCanvas couplingspanel;
	private Label x_bar;
	private Label y_bar;
	private Label coupling_bar;


	/** Listener for the file open dialog. */
	public class FileOpenListener implements Listener {
		/*
		 *  (non-Javadoc)
		 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
		 */
		public void handleEvent(Event e) {
			FileDialog chooser = new FileDialog(mainframe);
			chooser.setText("Select CSV or GML File To Open");
			String filename = chooser.open();
			if (filename != null) {
				try {
					SparePixMap pm = openFile(filename);
					if (pm==null) {
						printNotImplemented();
					}
					else {
						couplingspanel.setPixMap(pm);
					}
				}
				catch (IOException e1) {
					MessageBox messageBox =
						new MessageBox(mainframe, SWT.ICON_ERROR | SWT.OK);
					messageBox.setMessage("File cannot be loaded");
					messageBox.open();
				}
			}
		}
	}

	/** Listener fpr the file save dialog. */
	public class FileSaveListener implements Listener {

		public static final int PNG_FILE = 0;
		public static final int MAP_FILE = 1;
		public static final int GML_FILE = 2;

		private int mode;

		/** Create new file save listener.
		 * 
		 * @param a_mode either PNG_FILE, MAP_FILE, or GML_FILE.
		 */
		public FileSaveListener(int a_mode) {
			super();
			mode = a_mode;
		}

		/*
		 *  (non-Javadoc)
		 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
		 */
		public void handleEvent(Event e) {
			if (couplingspanel.getPixMap()==null) return;
			FileDialog chooser = new FileDialog(mainframe);
			chooser.setText("Select File To Save To");
			String filename = chooser.open();
			if (filename == null) {
				return;
			}

			if (mode == PNG_FILE) {
				try {
					couplingspanel.exportToPNG(filename);
				}
				catch (IOException e1) {
					MessageBox messageBox =
						new MessageBox(mainframe, SWT.ICON_ERROR | SWT.OK);
					messageBox.setMessage("File cannot be written");
					messageBox.open();
					System.out.println(filename + " not writable.");
				}
			}
			else if (mode == MAP_FILE) {
				try {
					BufferedWriter wr =
						new BufferedWriter(new FileWriter(filename));
					String[] myitems = couplingspanel.getItems();
					for (int i = 0; i < myitems.length; i++) {
						wr.write(myitems[i] + "\n");
					}
					wr.close();
				}
				catch (IOException e1) {
					MessageBox messageBox =
						new MessageBox(mainframe, SWT.ICON_ERROR | SWT.OK);
					messageBox.setMessage("File cannot be written");
					messageBox.open();
				}
			}
			else if (mode == GML_FILE) {
				try {
					couplingspanel.getPixMap().exportToGML(filename);
				} catch (IOException e1) {
					MessageBox messageBox =
						new MessageBox(mainframe, SWT.ICON_ERROR | SWT.OK);
					messageBox.setMessage("File cannot be written");
					messageBox.open();
				}
			}
			else {
				printNotImplemented();
			}
		}
	}

	/** Listener for chaninging the clipping area. */
	public class ViewClippingAreaListener implements Listener {
		public static final int AREA_START = 0;
		public static final int AREA_LENGTH = 1;
		
		private int mode;
		
		/** Create new view clipping area listener.
		 * 
		 * @param _mode either AREA_START or AREA_LENGTH
		 */
		public ViewClippingAreaListener(int _mode) {
			mode = _mode;
		}
		
		/*
		 *  (non-Javadoc)
		 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
		 */
		public void handleEvent(Event e) {
			if (couplingspanel.getPixMap()==null) return;
			switch (mode) {
			case AREA_START:
				IntegerSelectionDialog dialog =
					new IntegerSelectionDialog(mainframe, 0, couplingspanel.getPixMap().size()-couplingspanel.getClippingAreaLength(), couplingspanel.getClippingAreaStart());
				Integer newareastart = dialog.open();
				if (newareastart != null) {
					couplingspanel.setClippingAreaStart(newareastart.intValue());
					//newzoom = newzoomint.intValue();
				}
				break;
			}
		}
	}
	
	/** Listener for zooming actions. */
	public class ViewZoomListener implements Listener {

		public static final int ZOOM_IN = 0;
		public static final int ZOOM_OUT = 1;
		public static final int ZOOM_ADJUST = 2;

		private int mode;

		/** Create new view zoom listener.
		 * 
		 * @param a_mode either ZOOM_IN, ZOOM_OUR, or ZOOM_ADJUST
		 */
		public ViewZoomListener(int a_mode) {
			mode = a_mode;
		}

		/*
		 *  (non-Javadoc)
		 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
		 */
		public void handleEvent(Event e) {
			if (couplingspanel.getPixMap()==null) return;
			int oldzoom = couplingspanel.getZoom();
			int newzoom = oldzoom;
			switch (mode) {
				case ZOOM_IN :
					newzoom++;
					break;
				case ZOOM_OUT :
					newzoom--;
					if (newzoom < 1) {
						newzoom = 1;
					}
					break;
				case ZOOM_ADJUST :
					IntegerSelectionDialog dialog =
						new IntegerSelectionDialog(mainframe, 1, 20, oldzoom);
					Integer newzoomint = dialog.open();
					if (newzoomint != null) {
						newzoom = newzoomint.intValue();
					}
					break;
				default :
					printNotImplemented();
			}

			if (newzoom != oldzoom) {
				couplingspanel.setZoom(newzoom);
			}

		}
	}

	/** Listener for exiting the software */
	public class FileExitListener implements Listener {
		public void handleEvent(Event e) {
			System.exit(0);
		}
	}

	/** Listener for showing the About dialog. */
	public class HelpAboutListener implements Listener {
		public void handleEvent(Event e) {
			AboutDialog about = new AboutDialog(mainframe);
			about.open();
		}
	}
	
	/** Listener for selecting the metric to be viewed. */
	public class MetricSelectListener implements Listener {
		
		public static final int METRIC_CONFIDENCE=0;
		public static final int METRIC_SUPPORT=1;
		
		private int mode;
		
		/** Create new metric select listener
		 * 
		 * @param a_mode either METRIC_CONFIDENCE or METRIC_SUPPORT
		 */
		public MetricSelectListener(int a_mode) {
			super();
			mode=a_mode;
		}
		
		/*
		 *  (non-Javadoc)
		 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
		 */
		public void handleEvent(Event e) {
			if (couplingspanel.getPixMap()==null) return;
			switch (mode) {
				case METRIC_CONFIDENCE:
					couplingspanel.setDisplayedMetric(PixMapCanvas.PIXMAP_SHOW_CONFIDENCE);
					break;
				case METRIC_SUPPORT:
					couplingspanel.setDisplayedMetric(PixMapCanvas.PIXMAP_SHOW_SUPPORT);
					break;
			}
		}
	}

	/** Listener for color selections. */
	public class ColorSelectListener implements Listener {

		public static final int BACKGROUND_COLOR = 2;
		public static final int EMPTY_FIELD_COLOR = 3;
		public static final int CONFIDENCE_COLORSCALE = 4;

		private int mode;

		/** Create new color select listener.
		 * 
		 * @param a_mode either BACKGROUND_COLOR, EMPTY_FIELD_COLOR, 
		 *               or CONFIDENCE_COLORSCALE
		 */
		public ColorSelectListener(int a_mode) {
			super();
			mode = a_mode;
		}

		/*
		 *  (non-Javadoc)
		 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
		 */
		public void handleEvent(Event e) {
			if (couplingspanel.getPixMap()==null) return;
			switch (mode) {
				case BACKGROUND_COLOR :
					ColorDialog colorDialog4 = new ColorDialog(mainframe);
					colorDialog4.setText("Choose New Background Color");
					colorDialog4.setRGB(couplingspanel.getBackgroundColor());
					RGB selectedColor4 = colorDialog4.open();
					if (selectedColor4 != null) {
						couplingspanel.setBackgroundColor(selectedColor4);
					}
					break;
				case EMPTY_FIELD_COLOR :
					ColorDialog colorDialog5 = new ColorDialog(mainframe);
					colorDialog5.setText("Choose New Color For Fields With Value 0");
					colorDialog5.setRGB(couplingspanel.getBackgroundColor());
					RGB selectedColor5 = colorDialog5.open();
					if (selectedColor5 != null) {
						couplingspanel.setEmptyFieldColor(selectedColor5);
					}
					break;	
				case CONFIDENCE_COLORSCALE :
					ColorScaleDialog dialog = new ColorScaleDialog(mainframe);
					RGB[] selectedScale = dialog.open();
					if (selectedScale != null) {
						couplingspanel.setConfidenceColorScale(selectedScale);
					}
					break;
				default :
					printNotImplemented();
			}
		}
	}

	/** Listener for filter operations. */
	public class FilterListener implements Listener {

		public static final int MINSUP = 0;
		public static final int MINCONF = 1;
		public static final int PIXMAP = 2;
		public static final int NORMSUP = 3;

		private int mode;

		/** Create new filter listener.
		 * 
		 * @param a_mode either MINSUP, MINCONF, PIXMAP, or NORMSUP
		 */
		public FilterListener(int a_mode) {
			super();
			mode = a_mode;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
		 */
		public void handleEvent(Event e) {
			if (couplingspanel.getPixMap() == null)
				return;
			switch (mode) {
			case MINSUP:
				IntegerSelectionDialog dialog = new IntegerSelectionDialog(
						mainframe, 1, couplingspanel.getMaxSupportCount(),
						couplingspanel.getMinSupportCount());
				Integer value = dialog.open();
				if (value != null) {
					couplingspanel.setMinSupportCount(value.intValue());
				}
				break;
			case MINCONF:
				ValueSelectionDialog valdialog = new ValueSelectionDialog(
						mainframe);
				String val = valdialog.open();
				if (val != null) {
					try {
						double mcnew = Double.parseDouble(val);
						if ((mcnew >= 0.0) && (mcnew <= 1.0)) {
							couplingspanel.setMinConfidence(mcnew);
						} else {
							MessageBox messageBox = new MessageBox(mainframe,
									SWT.ICON_ERROR | SWT.OK);
							messageBox
									.setMessage("The given value must be a real number between 0 and 1.");
							messageBox.open();
						}
					} catch (NumberFormatException e1) {
						MessageBox messageBox = new MessageBox(mainframe,
								SWT.ICON_ERROR | SWT.OK);
						messageBox
								.setMessage("The given value must be a real number between 0 and 1.");
						messageBox.open();
					}
				}
				break;
			case PIXMAP:
				FileDialog chooser = new FileDialog(mainframe);
				chooser.setText("Select CSV File To Open");
				String filename = chooser.open();
				if (filename != null) {
					try {
						SparePixMap pm2 = openFile(filename);
						SparePixMap pm = couplingspanel.getPixMap();
						pm.getDifferencesTo(pm2);
						couplingspanel.setPixMap(pm);
					} catch (IOException e1) {
						MessageBox messageBox = new MessageBox(mainframe,
								SWT.ICON_ERROR | SWT.OK);
						messageBox.setMessage("File cannot be loaded");
						messageBox.open();
					}
				}
				break;
			case NORMSUP:
				IntegerSelectionDialog dialog_normsup = new IntegerSelectionDialog(
						mainframe, 1, couplingspanel.getMaxSupportCount(),
						couplingspanel.getNormalizeToSupportCount());
				Integer value_normsup = dialog_normsup.open();
				if (value_normsup != null) {
					couplingspanel.setNormalizeToSupportCount(value_normsup
							.intValue());
				}
				break;
			default:
				printNotImplemented();
			}
		}
	}
	
	/** Listener for mouse motions. */
	public class MyMouseMoveListener implements MouseMoveListener {

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.swt.events.MouseMoveListener#mouseMove(org.eclipse.swt.events.MouseEvent)
		 */
		public void mouseMove(MouseEvent e) {
			 
			int itematx = couplingspanel.getItemNumber(e.x);
			int itematy = couplingspanel.getItemNumber(e.y-1); // don't know why we need the "-1", but is works! 
			if ((itematx>-1) && (itematy>-1)) {
				x_bar.setText("x: "+itematx + " = "+ couplingspanel.getItems()[itematx]);
				y_bar.setText("y: "+itematy + " = " + couplingspanel.getItems()[itematy]);
				coupling_bar.setText("Coupling Strength: Support Count = "+couplingspanel.getSupportCount(itematx, itematy)+ "; Confidence = "+couplingspanel.getConfidence(itematx, itematy));
			}
		}
		
	}
	

	/** Open a new file and return the pixelmap in it.
	 * 
	 * @param filename name of the file
	 * @return pixelmap
	 * 
	 */
	private SparePixMap openFile(String filename) throws IOException {
		String prefix = "";
	
		if ((filename.toLowerCase().endsWith(".gml")) || (filename.toLowerCase().endsWith(".gml.gz"))) {
			SparePixMap pm = SparePixMap.fromGMLFile(filename,filename+".filter");
			return pm;
		}	else if (filename.toLowerCase().endsWith(".csv")) {
            int suffixpos1 = filename.lastIndexOf('.');
            String prefix_help = filename.substring(0,suffixpos1);
            int suffixpos2 = prefix_help.lastIndexOf('.');
            prefix = filename.substring(0,suffixpos2);
            String conffile = prefix + ".relative.csv";
            String supfile = prefix + ".absolute.csv";
            String mapfile = prefix + ".map";
            String filterfile = prefix + ".filter";
            SparePixMap pm = SparePixMap.fromCSVFiles(supfile, conffile, mapfile, filterfile);
            return pm;
        }   else if (filename.toLowerCase().endsWith(".csv.gz")) {
			int suffixpos1 = filename.lastIndexOf('.');
			String prefix_help = filename.substring(0,suffixpos1);
			int suffixpos2 = prefix_help.lastIndexOf('.');
			prefix_help = filename.substring(0,suffixpos2);
			int suffixpos3 = prefix_help.lastIndexOf('.');
			prefix = filename.substring(0,suffixpos3);
			String conffile = prefix + ".relative.csv.gz";
			String supfile = prefix + ".absolute.csv.gz";
			String mapfile = prefix + ".map.gz";
			String filterfile = prefix + ".filter";
			SparePixMap pm = SparePixMap.fromCSVFiles(supfile, conffile, mapfile, filterfile);
			return pm;
        }   else if (filename.toLowerCase().endsWith(".txt")) {
            String conffile = filename;
            String supfile = null;
            String mapfile = null;
            String filterfile = null;
            SparePixMap pm = SparePixMap.fromTXTFiles(supfile, conffile, mapfile, filterfile);
            return pm;
		} else {
			return null;
		}
	}

	/** Constructor of the main class of the software.
	 * It shows the user interface and lets the user do actions.
	 */ 
	public EPOSpix(Display maindis, SparePixMap initialpixmap) {
		//Display maindis = new Display();
		mainframe = new Shell(maindis);
		mainframe.setText("EPOSpix");
		FormLayout mainframelayout = new FormLayout();
		mainframe.setLayout(mainframelayout);

		Menu menu = new Menu(mainframe, SWT.BAR);
		mainframe.setMenuBar(menu);

		// File Menu
		MenuItem filemenuitem = new MenuItem(menu, SWT.CASCADE);
		filemenuitem.setText("&File");

		Menu filemenu = new Menu(mainframe, SWT.DROP_DOWN);
		filemenuitem.setMenu(filemenu);
		MenuItem file_open = new MenuItem(filemenu, SWT.PUSH);
		file_open.setText("&Open...\tCtrl+O");
		file_open.setAccelerator(SWT.CTRL + 'O');
		file_open.addListener(SWT.Selection, new FileOpenListener());

		MenuItem file_save = new MenuItem(filemenu, SWT.PUSH);
		file_save.setText("&Save pixmap to GML...\tCtrl+S");
		file_save.setAccelerator(SWT.CTRL + 'S');
		file_save.addListener(
			SWT.Selection,
			new FileSaveListener(FileSaveListener.GML_FILE));
		
		MenuItem file_export = new MenuItem(filemenu, SWT.PUSH);
		file_export.setText("&Export To PNG/JPG/BMP...\tCrtl+E");
		file_export.setAccelerator(SWT.CTRL + 'E');
		file_export.addListener(
			SWT.Selection,
			new FileSaveListener(FileSaveListener.PNG_FILE));

		MenuItem file_mapsave = new MenuItem(filemenu, SWT.PUSH);
		file_mapsave.setText("&Save Item Map...\tCtrl+M");
		file_mapsave.setAccelerator(SWT.CTRL + 'M');
		file_mapsave.addListener(
			SWT.Selection,
			new FileSaveListener(FileSaveListener.MAP_FILE));

		MenuItem file_exit = new MenuItem(filemenu, SWT.PUSH);
		file_exit.setText("E&xit\tAlt+F4");
		file_exit.setAccelerator(SWT.ALT + SWT.F4);
		file_exit.addListener(SWT.Selection, new FileExitListener());

		// View Menu
		MenuItem view_menuitem = new MenuItem(menu, SWT.CASCADE);
		view_menuitem.setText("&View");
		Menu view_menu = new Menu(mainframe, SWT.DROP_DOWN);
		view_menuitem.setMenu(view_menu);
		
		MenuItem view_menu_clipping = new MenuItem(view_menu, SWT.CASCADE);
		view_menu_clipping.setText("Set &Clipping Area...");
		view_menu_clipping.addListener( SWT.Selection, new ViewClippingAreaListener(ViewClippingAreaListener.AREA_START));	
		MenuItem view_menu_zoomitem = new MenuItem(view_menu, SWT.CASCADE);
		view_menu_zoomitem.setText("&Zoom");
		Menu view_menu_zoom = new Menu(mainframe, SWT.DROP_DOWN);
		view_menu_zoomitem.setMenu(view_menu_zoom);
		MenuItem view_zoomin = new MenuItem(view_menu_zoom, SWT.PUSH);
		view_zoomin.setText("Zoom &In\t+");
		view_zoomin.setAccelerator('+');
		view_zoomin.addListener(
			SWT.Selection,
			new ViewZoomListener(ViewZoomListener.ZOOM_IN));

		MenuItem view_zoomout = new MenuItem(view_menu_zoom, SWT.PUSH);
		view_zoomout.setText("Zoom &Out\t-");
		view_zoomout.setAccelerator('-');
		view_zoomout.addListener(
			SWT.Selection,
			new ViewZoomListener(ViewZoomListener.ZOOM_OUT));

		MenuItem view_zoom = new MenuItem(view_menu_zoom, SWT.PUSH);
		view_zoom.setText("Adjust &Zoom...\tCtrl+Z");
		view_zoom.setAccelerator(SWT.CTRL + 'Z');
		view_zoom.addListener(
			SWT.Selection,
			new ViewZoomListener(ViewZoomListener.ZOOM_ADJUST));
		
		MenuItem view_menu_metricitem = new MenuItem(view_menu, SWT.CASCADE);
		view_menu_metricitem.setText("Select &Metric");
		Menu view_menu_metric = new Menu(mainframe, SWT.DROP_DOWN);
		view_menu_metricitem.setMenu(view_menu_metric);
		MenuItem view_metric_confidence = new MenuItem(view_menu_metric, SWT.PUSH);
		view_metric_confidence.setText("Display &Confidence");
		view_metric_confidence.addListener(
			SWT.Selection,
			new MetricSelectListener(MetricSelectListener.METRIC_CONFIDENCE));
		MenuItem view_metric_support = new MenuItem(view_menu_metric, SWT.PUSH);
		view_metric_support.setText("Display &Support");
		view_metric_support.addListener(
			SWT.Selection,
			new MetricSelectListener(MetricSelectListener.METRIC_SUPPORT));

		MenuItem view_color_subitem = new MenuItem(view_menu, SWT.CASCADE);
		view_color_subitem.setText("Change &Colors");
		Menu view_color_sub = new Menu(mainframe, SWT.DROP_DOWN);
		view_color_subitem.setMenu(view_color_sub);
		MenuItem view_selectconfcolor = new MenuItem(view_color_sub, SWT.PUSH);
		view_selectconfcolor.setText("&Metric Color Scale...");
		view_selectconfcolor.addListener(
			SWT.Selection,
			new ColorSelectListener(ColorSelectListener.CONFIDENCE_COLORSCALE));
		MenuItem view_selectemptycolor = new MenuItem(view_color_sub, SWT.PUSH);
		view_selectemptycolor.setText("&Empty Field Color...");
		view_selectemptycolor.addListener(
			SWT.Selection,
			new ColorSelectListener(ColorSelectListener.EMPTY_FIELD_COLOR));
		MenuItem view_selectbgcolor = new MenuItem(view_color_sub, SWT.PUSH);
		view_selectbgcolor.setText("&Background Color...");
		view_selectbgcolor.addListener(
			SWT.Selection,
			new ColorSelectListener(ColorSelectListener.BACKGROUND_COLOR));

		// Tools menu
		MenuItem tools_menuitem = new MenuItem(menu, SWT.CASCADE);
		tools_menuitem.setText("&Tools");
		Menu tools_menu = new Menu(mainframe, SWT.DROP_DOWN);
		tools_menuitem.setMenu(tools_menu);

		MenuItem tools_filteritem = new MenuItem(tools_menu, SWT.CASCADE);
		tools_filteritem.setText("&Filter");
		Menu tools_filter = new Menu(mainframe, SWT.DROP_DOWN);
		tools_filteritem.setMenu(tools_filter);
		MenuItem tools_filter_sup = new MenuItem(tools_filter, SWT.PUSH);
		tools_filter_sup.setText("Minimal &Support...");
		tools_filter_sup.addListener(
			SWT.Selection,
			new FilterListener(FilterListener.MINSUP));
		MenuItem tools_filter_conf = new MenuItem(tools_filter, SWT.PUSH);
		tools_filter_conf.setText("Minimal &Confidence...");
		tools_filter_conf.addListener(
			SWT.Selection,
			new FilterListener(FilterListener.MINCONF));
		MenuItem tools_filter_normsup = new MenuItem(tools_filter, SWT.PUSH);
		tools_filter_normsup.setText("Normalize to a certain support value...");
		tools_filter_normsup.addListener(
				SWT.Selection,
				new FilterListener(FilterListener.NORMSUP));
		
		MenuItem tools_filter_pixmap = new MenuItem(tools_filter, SWT.PUSH);
		tools_filter_pixmap.setText("Difference To &Pixmap...");
		tools_filter_pixmap.addListener(
			SWT.Selection,
			new FilterListener(FilterListener.PIXMAP));
		
		//
		// Help menu
		MenuItem help_menuitem = new MenuItem(menu, SWT.CASCADE);
		help_menuitem.setText("&Help");
		Menu help_menu = new Menu(mainframe, SWT.DROP_DOWN);
		help_menuitem.setMenu(help_menu);
		MenuItem help_about = new MenuItem(help_menu, SWT.PUSH);
		help_about.setText("&About...");
		help_about.addListener(SWT.Selection, new HelpAboutListener());

		ScrolledComposite sc2 =
			new ScrolledComposite(
				mainframe,
				SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER);
		couplingspanel = new PixMapCanvas(sc2, SWT.NONE);
		couplingspanel.addMouseMoveListener(new MyMouseMoveListener());
        if (initialpixmap != null) couplingspanel.setPixMap(initialpixmap);
		sc2.setContent(couplingspanel);
		FormData data2 = new FormData();
		data2.top = new FormAttachment(0, 0);
		data2.bottom = new FormAttachment(100, -60);
		data2.left = new FormAttachment(0, 0);
		data2.right = new FormAttachment(100, 0);
		sc2.setLayoutData(data2);

		x_bar = new Label(mainframe, SWT.BORDER);
		x_bar.setText(y_bar_empty_text);
		FormData data1 = new FormData();
		data1.height = 15;
		data1.bottom = new FormAttachment(100, -40);
		data1.top = new FormAttachment(100, -61);
		data1.left = new FormAttachment(0, 0);
		data1.right = new FormAttachment(100, 0);
		x_bar.setLayoutData(data1);

		y_bar = new Label(mainframe, SWT.BORDER);
		y_bar.setText(x_bar_empty_text);
		FormData data3 = new FormData();
		data3.height = 15;
		data3.bottom = new FormAttachment(100, -20);
		data3.top = new FormAttachment(100, -41);
		data3.left = new FormAttachment(0, 0);
		data3.right = new FormAttachment(100, 0);
		y_bar.setLayoutData(data3);
		
		coupling_bar = new Label(mainframe, SWT.BORDER);
		coupling_bar.setText(coupling_bar_empty_text);
		FormData data4 = new FormData();
		data4.height = 15;
		data4.bottom = new FormAttachment(100,0);
		data4.top = new FormAttachment(100, -21);
		data4.left = new FormAttachment(0, 0);
		data4.right = new FormAttachment(100, 0);
		coupling_bar.setLayoutData(data4);

		mainframe.setSize(800,600);

		

		mainframe.open();
		//while (!mainframe.isDisposed())
		//	if (!maindis.readAndDispatch())
		//		maindis.sleep();
		//maindis.dispose();

	}

	/** Print the "not implemented" message.
	 *
	 */
	private void printNotImplemented() {
		MessageBox messageBox =
			new MessageBox(mainframe, SWT.ICON_WARNING | SWT.OK);
		messageBox.setMessage("This function is not implemented.");
		messageBox.open();
	}

	public static void main(String[] args) {

		new EPOSpix(new Display(), null);

	}
}
