/*
 * $Id: DatabaseOperations.java,v 1.7 2004/12/28 21:39:28 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.assist.mining.db;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;

import org.softevo.rose.assist.AssistPlugin;
import org.softevo.rose.db.DBConnectionProperties;
import org.softevo.rose.db.DatabaseProvider;
import org.softevo.rose.db.jdbc.DatabaseProperties;
import org.softevo.util.StringUtils;


/**
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class DatabaseOperations {

        
    private Connection connection = null;
    private DBConnectionProperties properties;
    
    private final String INT;
    private final String VARCHAR;
    private final String TIMESTAMP;
    private Statement stmt;
    private DatabaseProvider provider;
    private PreparedStatement stmtActivities;
    private PreparedStatement stmtLineitems;
    
    public DatabaseOperations(DatabaseProvider provider) {
        this.provider = provider;

        properties = provider.getConnectionProperties();
        INT = properties.getDatabaseProperties().getTypeMapping(DatabaseProperties.TP_INT);
        VARCHAR = properties.getDatabaseProperties().getTypeMapping(DatabaseProperties.TP_VARCHAR);
        TIMESTAMP = properties.getDatabaseProperties().getTypeMapping(DatabaseProperties.TP_TIMESTAMP);

    }
    

    public void init() throws SQLException {
        connection = provider.getConnection();
        stmt = connection.createStatement();  
    }
    
    public void setupStmts() throws SQLException {
        stmtActivities = createInsertStatement(getTableName(Tables.TABLE_ACTIVITIES), 4);
        stmtLineitems = createInsertStatement(getTableName(Tables.TABLE_LINEITEMS), 5);
    }
    
    public void close() throws SQLException {
        stmtActivities.close();
        stmtLineitems.close();
        stmt.close();
    }
    
    
    public void addConstraintUnique
    	(String constraintName, String tableName, String[] columnList) 
    	throws SQLException {
        
		String query = "ALTER TABLE " + tableName + " "
			+ "ADD CONSTRAINT " + constraintName + " "
			+ "UNIQUE (" + StringUtils.join(columnList, ",") + ");";
		executeUpdate(query);		
    }
    
    
    public void addConstraintForeignKey
    	(String constraintName, String tableName, String[] columnList, 
    	 String targetTable, String[] targetColumnList) throws SQLException {
        
		String query = "ALTER TABLE " + tableName + " "
			+ "ADD CONSTRAINT " + constraintName + " "
			+ "FOREIGN KEY (" + StringUtils.join(columnList, ",") + ") "
			+ "REFERENCES " + targetTable 
				+ " (" + StringUtils.join(targetColumnList, ",") + ") "
				+ "ON UPDATE CASCADE";
		executeUpdate(query);	
    }
    

	/**
	 * TODO Avoid Action, When,etc
	 */
	public void createTableActivities(String tableName) throws SQLException {		
	    tableName = tableName == null ? getTableName(Tables.TABLE_ACTIVITIES) : tableName;		
		String query = "CREATE TABLE " + tableName + " ( "
			+ "EntityID " + INT + " NOT NULL, "
			+ "Action " + VARCHAR + "(6) NOT NULL, "
			+ "Activity_When " + TIMESTAMP + " NOT NULL, "
			+ "Branch " + VARCHAR + "(50) " //, "
			//+ "CONSTRAINT " + tableName + "_pkey "
			//	+ "PRIMARY KEY (EntityID, ActivityWhen, Branch) "
			+ ");";
		executeUpdate(query);

		// TODO createIndex
		query = "CREATE INDEX " + tableName + "_idx " + "ON "	+ tableName + "(EntityID)";
		executeUpdate(query);
	}

	
	/**
	 * 
	 * @throws SQLException
	 */
	public void createTableLineitems(String tableName) throws SQLException {		
	    tableName = tableName == null ? getTableName(Tables.TABLE_LINEITEMS) : tableName;		
		String query = "CREATE TABLE " + tableName + " ( "
			+ "TransactionID " + INT + " NOT NULL, "
			+ "EntityID " + INT + " NOT NULL, "
			+ "Action " + VARCHAR + "(6) NOT NULL, "
			+ "Lineitem_When " + TIMESTAMP + " NOT NULL, "
			+ "IsNoise " + VARCHAR + "(1) NOT NULL "
			+ ");";
		executeUpdate(query);	
		
		// TODO createIndex
		query = "CREATE INDEX " + tableName + "_idx " + "ON "	+ tableName + "(TransactionID, EntityID)";
	}

	
	
	
	/**
	 * 
	 * @param tableName
	 * @throws SQLException
	 */
	public void dropTable(String tableName) throws SQLException {		
		String query = "DROP TABLE " + tableName + ";";
	    executeUpdate(query);
	}

	
	/**
	 * 
	 * @param indexName
	 * @throws SQLException
	 */
	public void dropIndex(String indexName) throws SQLException {		
		String query = "DROP INDEX " + indexName + ";";
	    executeUpdate(query);
	}

	
	public void dropConstraint(String tableName, String constraintName) 
		throws SQLException {
	    
		String query = "ALTER TABLE " + tableName + " "
			+ "DROP CONSTRAINT " + constraintName; // + "RESTRICT";
		executeUpdate(query);
	}
	
	/**
	 * Tests whether a table exists. 
	 * @param tableName
	 * @return boolean
	 */
	public boolean existsTable(String tableName) {
	    // Use "SELECT COUNT(*) FROM " + tableName + ";";
	    // But this should be faster (except for braindead databases):
	    String query = "SELECT 1 FROM " + tableName + " WHERE 1=0;";
	    try {
            stmt.executeQuery(query);
        } catch (SQLException e) {
            return false;
        }
	    return true;
	}
	

	public String getTableName(String tableID) {
	    return Tables.PREFIX_CVS + tableID;
	}

	
	/**
	 * @throws SQLException
	 */
	public void insertActivity(int entityID, String action, Timestamp when, String branch) 
		throws SQLException {
	
	    stmtActivities.setInt(1, entityID);
	    stmtActivities.setString(2, action);
	    stmtActivities.setTimestamp(3, when);
	    stmtActivities.setString(4, branch);
	    stmtActivities.execute();	    
	}

	/**
	 * @throws SQLException
	 */
	public void insertLineitem
		(int transactionID, int entityID, String action, Timestamp when, boolean isNoise) 
		throws SQLException {
	
	    stmtLineitems.setInt(1, transactionID);
	    stmtLineitems.setInt(2, entityID);
	    stmtLineitems.setString(3, action);
	    stmtLineitems.setTimestamp(4, when);
	    stmtLineitems.setString(5, isNoise ? "Y" : "N");
	    stmtLineitems.execute();	    
	}

	
	public void renameTable(String oldName, String newName) 
		throws SQLException {
	    
		String query = "ALTER TABLE " + oldName	+ " RENAME TO " + newName;
		executeUpdate(query);
	}
	
	
	public void setAutoCommit(boolean autoCommit) throws SQLException {
	    connection.setAutoCommit(autoCommit);
	}
	
		
	private PreparedStatement createInsertStatement(String tableName, int numberOfFields) throws SQLException {
	    StringBuffer query = new StringBuffer("INSERT INTO " + tableName + " VALUES (");
	    for (int i = 0; i < numberOfFields; i++) {
            if (i>0) {
                query.append(",?");
            } else {
                query.append("?");
            }
        }
	    query.append(");");
	    PreparedStatement result = connection.prepareStatement(query.toString());
		return result;
	}	

	
	public void executeUpdate(String query) throws SQLException {
	    AssistPlugin.debugInfo(query);
		stmt.executeUpdate(query);	    
	}

	public ResultSet executeQuery(String query) throws SQLException {
	    return stmt.executeQuery(query);
	}

}
