/*
 * $Id: LogInfo.java,v 1.3 2005/01/04 13:11:11 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.ccvs.model;

import java.io.Serializable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.softevo.util.StringUtils;

/**
 * This class represents the output of a CVS log command.
 * <P>
 * It consists of a 
 * <UL>
 * <LI> <EM>fileName</EM>, for which the log operation was called.
 * <LI> <EM>selectRevisions</EM>, the number of selected revision.
 * <LI> <EM>totalRevisions</EM>, the total number of all revisions.
 * <LI> <EM>keywordExpansion</EM>, usually an indicator for binary files.
 * <LI> list of <EM>revisions</EM>.
 * <LI> list of <EM>branches</EM>.
 * <LI> list of <EM>tags</EM>.
 * </UL>
 * 
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class LogInfo implements Serializable {
	
    /** The serialization ID */
	private static final long serialVersionUID = -3843617192970007176L;

	/** The filename */
    private String fileName;
	
    /** The list of revisions */
	private LinkedList revisions = null;
    /** The list of branches */
	private LinkedList branches = null;
    /** The list of tags */
	private LinkedList tags = null;	
	
	/** The number of selected revisions */
	private int selectedRevisions = -1;
	/** The total number of revision */
	private int totalRevisions = -1;
	/** The keyword expansion */
	private String keywordExpansion = null;
	
	/**
	 * Creates a LogInfo object for the given file.
	 * 
	 * @param fileName  the filename.
	 */
	public LogInfo(String fileName) {
	    // Remove superfluous / (by replacing at least 2 / by a single /)
		this.fileName = fileName.replaceAll("/{2,}", "/");

		revisions = new LinkedList();
		tags = new LinkedList();
		branches = new LinkedList();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		
		StringBuffer result = new StringBuffer();
		String linesep = System.getProperty("line.separator");
		
		result.append("Filename = " + fileName + "  [" + keywordExpansion+ "]");
		result.append(linesep);
		result.append("  revisions selected = " + selectedRevisions);
		result.append("  total = " + totalRevisions);
		result.append(linesep);

		result.append(linesep);
		result.append("** Revisions **" + linesep);
		for (Iterator iter = revisions.iterator(); iter.hasNext();) {
			RevisionItem element = (RevisionItem) iter.next();
			result.append(element.toString());
			result.append(linesep);
		}

		result.append(linesep);
		result.append("** Branches **" + linesep);
		for (Iterator iter = branches.iterator(); iter.hasNext();) {
			BranchItem element = (BranchItem) iter.next();
			result.append(element.toString());
			result.append(linesep);
		}

		result.append(linesep);
		result.append("** Tags **" + linesep);
		for (Iterator iter = tags.iterator(); iter.hasNext();) {
			TagItem element = (TagItem) iter.next();
			result.append(element.toString());
			result.append(linesep);
		}

		return result.toString();
	}

	/**
	 * Adds an element to the branch list.
	 * @param branch  the branch to be added.
	 * @return <code>true</code> (see Collection.add)
	 */
	public boolean addBranch(BranchItem branch) {
		return branches.add(branch);
	}

	/**
	 * Adds an element to the revision list.
	 * @param revision  the revision to be added.
	 * @return <code>true</code> (see Collection.add)
	 */
	public boolean addRevision(RevisionItem revision) {
		return revisions.add(revision);
	}

	/**
	 * Adds an element to the tag list.
	 * @param tag  the tag to be added.
	 * @return <code>true</code> (see Collection.add)
	 */
	public boolean addTag(TagItem tag) {
		return tags.add(tag);
	}
	

    /**
     * @return Returns the branches.
     */
    public LinkedList getBranches() {
        return branches;
    }

    /**
     * @return Returns the fileName.
     */
    public String getFileName() {
        return fileName;
    }

    /**
     * @return Returns the keywordExpansion.
     */
    public String getKeywordExpansion() {
        return keywordExpansion;
    }

    /**
     * @return Returns the revisions.
     */
    public LinkedList getRevisions() {
        return revisions;
    }

    /**
     * @return Returns the selectedRevisions.
     */
    public int getSelectedRevisions() {
        return selectedRevisions;
    }

    /**
     * @return Returns the tags.
     */
    public LinkedList getTags() {
        return tags;
    }

    /**
     * @return Returns the totalRevisions.
     */
    public int getTotalRevisions() {
        return totalRevisions;
    }

    /**
     * @param keywordExpansion The keywordExpansion to set.
     */
    public void setKeywordExpansion(String keywordExpansion) {
        this.keywordExpansion = keywordExpansion;
    }

    /**
     * @param selectedRevisions The selectedRevisions to set.
     */
    public void setSelectedRevisions(int selectedRevisions) {
        this.selectedRevisions = selectedRevisions;
    }

    /**
     * @param totalRevisions The totalRevisions to set.
     */
    public void setTotalRevisions(int totalRevisions) {
        this.totalRevisions = totalRevisions;
    }
    
    
	private static final Pattern FILE_PATTERN = 
	    Pattern.compile("(.*/)((.+)(\\.[^.]*)|(\\.?[^.]+))");

	public String getDirectoryName() {
	    Matcher m = FILE_PATTERN.matcher(fileName);
		if (m.matches()) {
			return m.group(1);
		}
		return null;
	}
	public String getFileShortName() {
	    Matcher m = FILE_PATTERN.matcher(fileName);
		if (m.matches()) {
			return m.group(2);
		}
		return null;
	}
	public String getFileExtension() {
	    Matcher m = FILE_PATTERN.matcher(fileName);
		if (m.matches()) {
			return m.group(4);
		}
		return null;
	}
	public int getFileDepth() {
	    return StringUtils.countFrequency(fileName, "/");
	}
    
}
