/*
 * $Id: DefaultLogParser.java,v 1.10 2005/03/01 12:55:26 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.ccvs.parser;

import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.runtime.Status;
import org.softevo.rose.ccvs.CCVSPlugin;
import org.softevo.rose.ccvs.model.BranchItem;
import org.softevo.rose.ccvs.model.LogInfo;
import org.softevo.rose.ccvs.model.RevisionItem;
import org.softevo.rose.ccvs.model.TagItem;
import org.softevo.util.FileUtils;


/**
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class DefaultLogParser implements ILogParser {

	/** Group definition for a cvs revision */
	private static final String CVS_REV = "([0-9\\.]+\\.[0-9]+)";

	/** Group definition for a date in a cvs log output */
	private static final String CVS_DATE_1_11 = 
		"[0-9]+/[0-9]+/[0-9]+ [0-9]+:[0-9]+:[0-9]+";
	private static final String CVS_DATE_1_12 = 
		"[0-9]+-[0-9]+\\-[0-9]+ [0-9]+:[0-9]+:[0-9]+ [ +-]?[0-9]+";
	private static final String CVS_DATE = 
		"(" + CVS_DATE_1_11 + "|" + CVS_DATE_1_12 + ")";

	/** Matches the "symbolic names"-line in a cvs log output. */
	private static final Pattern CVS_SYMBOLIC_NAMES =
		Pattern.compile("symbolic names:");
		
	/** 
	 * Matches a tag specification in a cvs log output. The first group 
	 * contains the tag name, the second group the corresponding revision.
	 */
	private static final Pattern CVS_TAG =
		Pattern.compile("\\s([^:]+): ([0-9\\.]+\\.[0-9]+)");
		
	/**
	 * Matches the keyword substitution line of a cvs log output.
	 * The group contains the specified substitution. With keyword 
	 * substitution it is possible to recognize the binary flag of a file.
	 */
	private static final Pattern CVS_KEYWORD_SUBST =
		Pattern.compile("keyword substitution: (.*)");
		
	/** 
	 * Matches the revision count line, of a cvs log output.
	 * The first group specifies the total number of revisions for this
	 * file, the secons group contains the number of selected revision.
	 * If everything works well, both groups are the same.
	 */
	private static final Pattern CVS_TOTAL_REVISIONS =
		Pattern.compile("total revisions: ([0-9]+);\\s" 
						+ "selected revisions: ([0-9]+)");

	/**
	 * Matches the description line of a cvs log output. This line
	 * marks the begin of the log messages part.
	 */
	private static final Pattern CVS_DESCRIPTION =
		Pattern.compile("description:");

	/**
	 * Matches a separation line of a cvs log output. This line is used 
	 * to separate log messages and to mark the end of the cvs log
	 * output. Unfortunately programmers use this line in their 
	 * messages, too. So don't rely on this Pattern and use an additional
	 * lookahead.
	 */
	private static final Pattern CVS_SEPARATOR = 
		Pattern.compile("-{28}|={77}");

	// private static final Pattern CVS_FINAL = Pattern.compile("={77}");
		
	/**
	 * Matches a revision line of a cvs log output. The first group
	 * will give use the revision. The second group contains information
	 * about locks existing for this revision.
	 */
	private static final Pattern CVS_REVISION_LINE = 
		Pattern.compile("revision " + CVS_REV + "(\\slocked by: .+;)?");
	
	/**
	 * Matches a branches line of a cvs log output. This line is used to 
	 * mark starting points of branches. If you find such a line, you know
	 * for sure that there exists a branch where this file was modified.
	 * The first group contains all branches for a revision separated by
	 * semikolons.
	 */
	private static final Pattern CVS_BRANCHES = 
		Pattern.compile("branches:  (.+)");
		
	/**
	 * Matches a cvs status line. This line is richof information: 
	 * The first group contains the checkin date, the second the commiter, 
	 * the third one the state in the rcs file, the fifth and sixth are
	 * optional and contain the number of added and removed lines. 
	 */
	private static final Pattern CVS_STATUS_LINE = 
		Pattern.compile("date: " + CVS_DATE + ";  author: ([^;]+);" 
			+ "  state: ([^;]+);(  lines:( \\+([0-9]+) -([0-9]+))?)?");

	//private static Category log =
	//    Category.getInstance(DefaultLogParser.class.getName());

	/**
	 * @see ILogParser#getLogInfo(String, String)
	 */
	public LogInfo getLogInfo(String fileName, String output) {

		DateFormat df = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
		DateFormat df2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss Z");
		String lineSeparator = System.getProperty("line.separator");
		
		LogInfo logInfo = new LogInfo(fileName);
			
		try {

		    // Parse cvs log information 
			RevisionItem revision = new RevisionItem();
			StringBuffer message = new StringBuffer();
			HashMap branchTags = new HashMap();
			int lineitem = 0;

			// Flags
			boolean matchRevisions = false;
			boolean matchTags = false;

			// for-loop over cvs log output
		    String[] lines = output.split(lineSeparator);

			for (int no = 0; no < lines.length; no++) {
                
			    String line = lines[no];
			    String lookahead = no < lines.length-1 ? lines[no+1] : null;

				if (matchRevisions) {
					// Already found a revision
					// Match information for a specific revision
					Matcher mSep = CVS_SEPARATOR.matcher(line);
					Matcher mRev = CVS_REVISION_LINE.matcher(line);
					Matcher mStatus = CVS_STATUS_LINE.matcher(line);
					Matcher mBranches = CVS_BRANCHES.matcher(line);

					/*** REVISIONS ***/
					if (mSep.matches()) {
						// Two cases: 
						// Real cvs separator or human entered separator

						if (lookahead == null  // end of cvs log reached
							|| Pattern.matches
								(CVS_REVISION_LINE.pattern(), lookahead)) {

							// Create a revision
							lineitem++;
							revision.setMessage(message.toString());
							revision.setPosition(lineitem);
							logInfo.addRevision(revision);

							// Reset variables for the new revision 
							revision = new RevisionItem();
							message = new StringBuffer();

						} else if (message != null) {
							// Just a text in the log message - append 
							// to message
							message.append(line + lineSeparator);
						}

					} else if (mRev.matches()) {
						if (revision.getRevision() == null) {
							// Matched the first revision line: get revision
							revision.setRevision(mRev.group(1));
						} else {
							// Just a text in the log message - append 
							// to message
							message.append(line + lineSeparator);							
						}

					} else if (mStatus.matches()) {
						// Matched a status line: 
						// get all information of the line
					    Date timestamp = null;
					    try {
					        timestamp = df.parse(mStatus.group(1));
					    } catch (ParseException e) {
					        timestamp = df2.parse(mStatus.group(1));
                        }
						revision.setDate(timestamp);
						revision.setAuthor(mStatus.group(2));
						revision.setState(mStatus.group(3));
						
						// group(6) and group(7) of m_status 
						// are both == null or both != null 
						if (mStatus.group(6) != null 
							&& mStatus.group(7) != null) {

							revision.setPlusLines
								(Integer.parseInt(mStatus.group(6)));
							revision.setMinusLines
								(-Integer.parseInt(mStatus.group(7)));
						} else {
							revision.setPlusLines(0);
							revision.setMinusLines(0);
						}

					/*** BRANCHES ***/
					} else if (mBranches.matches()) {
						// Matched a branch line: Create branches
						StringTokenizer st = 
							new StringTokenizer(mBranches.group(1), "; ");

						for (int i = 0; i < st.countTokens(); i++) {
						    String prefix = st.nextToken();
						    TagItem tag = (TagItem) branchTags.get(prefix);
						    if (tag!=null) {
								BranchItem branch = new BranchItem(
	                                    tag.getTagName(), prefix,
	                                    tag.getRevision(), 
	                                    revision.getRevision());
								logInfo.addBranch(branch);
						    } else if (!"1.1.1".equals(prefix)){
						        CCVSPlugin.logWarning("WARNING: Found no tag for branch " + prefix, null);
						    }
						}

					} else {
						// No special line: log message
						message.append(line + lineSeparator);
					}	

				} else { // if (!matchRevisions)
					// Still in the header of the cvs log message
					// Match information for the whole file
					Matcher mSep = CVS_SEPARATOR.matcher(line);
					Matcher mSymbolic = CVS_SYMBOLIC_NAMES.matcher(line);
					Matcher mTag = CVS_TAG.matcher(line);
					Matcher mExpand = CVS_KEYWORD_SUBST.matcher(line);
					Matcher mTotal = CVS_TOTAL_REVISIONS.matcher(line);
					Matcher mDescription = CVS_DESCRIPTION.matcher(line);

					/*** TAGS ***/
					if (matchTags && mTag.matches()) { 
					    String tagName = mTag.group(1);
					    String tagRevision = mTag.group(2);
						TagItem tag = new TagItem(tagName, tagRevision);

						if (tagRevision.indexOf(".0.")<0) {
							// Matched a real tag
							logInfo.addTag(tag);
						} else {
						    // Matched a tag for a branch
						    String branchPrefix = 
						        tagRevision.replaceFirst("\\.0\\.", "\\.");
						    branchTags.put(branchPrefix, tag);
						}

					/*** OTHER ***/
					} else if (mSymbolic.matches()) {
						// Match the "symbolic names" keyword
						matchTags = true;

					} else if (mExpand.matches()) {
						// Matched a keyword substitution line (binary tag)
						logInfo.setKeywordExpansion(mExpand.group(1));
						
						// This line marks end of tag section as well
						matchTags = false;

					} else if (mTotal.matches()) {
						// Matched a selected/total revision line
						logInfo.setTotalRevisions
							(Integer.parseInt(mTotal.group(1)));
						logInfo.setSelectedRevisions
							(Integer.parseInt(mTotal.group(2)));

					} else if (mDescription.matches()) {
                        // empty
					} else if (mSep.matches()) {
						// Found first revision
						matchRevisions = true;
					}
				}

			} // end for-loop over cvs log output
			
			if (!matchRevisions) {
			    CCVSPlugin.logWarning("WARNING: Found no revisions for " + fileName, null);
			}

		} catch (ParseException e) {			
		    CCVSPlugin.logWarning("WARNING: Failed to parse dates for " + fileName, null);
	        CCVSPlugin.logException(e, Status.WARNING);
			return null;
		}

		return logInfo;
	}
	
	
	/**
	 * Main method.
	 * <P>
	 * Test the DefaultLogParser with the files specified
	 * in args. Each file contains the output of one log command. 
	 * 
	 * @param args  the files to test.
	 */
	public static void main(String[] args) {
		ILogParser logParser = new DefaultLogParser();
		for (int i = 0; i < args.length; i++) {
	        try {
                String content = FileUtils.readFileToString(args[i], "UTF-8");
                System.out.println(logParser.getLogInfo(args[i], content));
            } catch (IOException e) {
                e.printStackTrace();
            }         
        }
    }
}
