/*
 * $Id: DatabaseOperations.java,v 1.9 2004/12/28 21:39:16 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.data.entities.db;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.softevo.rose.data.entities.EntitiesPlugin;
import org.softevo.rose.data.entities.model.ChangedEntity;
import org.softevo.rose.data.entities.model.Entity;
import org.softevo.rose.db.DBConnectionProperties;
import org.softevo.rose.db.DatabaseProvider;
import org.softevo.rose.db.jdbc.DatabaseProperties;
import org.softevo.util.StringUtils;


/**
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class DatabaseOperations {

        
    private Connection connection = null;
    private DBConnectionProperties properties;
    
    private final String INT;
    private final String TEXT;
    private final String VARCHAR;
    private Statement stmt;
    private DatabaseProvider provider;
    private PreparedStatement stmtEntities;
    private PreparedStatement stmtChangedEntities;
    
    public DatabaseOperations(DatabaseProvider provider) {
        this.provider = provider;

        properties = provider.getConnectionProperties();
        INT = properties.getDatabaseProperties().getTypeMapping(DatabaseProperties.TP_INT);
        TEXT = properties.getDatabaseProperties().getTypeMapping(DatabaseProperties.TP_TEXT);
        VARCHAR = properties.getDatabaseProperties().getTypeMapping(DatabaseProperties.TP_VARCHAR);

    }
    

    public void init() throws SQLException {
        connection = provider.getConnection();
        stmt = connection.createStatement();  
    }
    
    public void setupStmts() throws SQLException {
        stmtEntities = createInsertStatement(getTableName(Tables.TABLE_ENTITIES), 3);
        stmtChangedEntities = createInsertStatement(getTableName(Tables.TABLE_CHANGED_ENTITIES), 6);
    }
    
    public void close() throws SQLException {
        stmtEntities.close();
        stmtChangedEntities.close();
        stmt.close();
    }
    
    
    public void addConstraintUnique
    	(String constraintName, String tableName, String[] columnList) 
    	throws SQLException {
        
		String query = "ALTER TABLE " + tableName + " "
			+ "ADD CONSTRAINT " + constraintName + " "
			+ "UNIQUE (" + StringUtils.join(columnList, ",") + ");";
		executeUpdate(query);		
    }
    
    
    public void addConstraintForeignKey
    	(String constraintName, String tableName, String[] columnList, 
    	 String targetTable, String[] targetColumnList) throws SQLException {
        
		String query = "ALTER TABLE " + tableName + " "
			+ "ADD CONSTRAINT " + constraintName + " "
			+ "FOREIGN KEY (" + StringUtils.join(columnList, ",") + ") "
			+ "REFERENCES " + targetTable 
				+ " (" + StringUtils.join(targetColumnList, ",") + ") "
				+ "ON UPDATE CASCADE";
		executeUpdate(query);	
    }
    

	/**
	 * 
	 * @see softevo.database.DatabaseConnection#createTableEntities()
	 */
	public void createTableEntities(String tableName) throws SQLException {		
	    tableName = tableName == null ? getTableName(Tables.TABLE_ENTITIES) : tableName;		
		String query = "CREATE TABLE " + tableName + " ( "
			+ "EntityID " + INT + " NOT NULL, "
			+ "EntityName " + VARCHAR + "(600) NOT NULL, "
			+ "FileID " + INT + ", "
			+ "CONSTRAINT " + tableName + "_pkey "
				+ "PRIMARY KEY (EntityID) "
			+ ");";
		executeUpdate(query);

		// TODO Index required:
		//query = "CREATE INDEX " + tableName + "_idx " + "ON "	+ tableName + "(FileID)";
	}

	
	/**
	 * 
	 * @throws SQLException
	 * @see softevo.database.DatabaseConnection#createTableModifiedSymbols(String)
	 */
	public void createTableChangedEntities(String tableName) throws SQLException {		
	    tableName = tableName == null ? getTableName(Tables.TABLE_CHANGED_ENTITIES) : tableName;		
		String query = "CREATE TABLE " + tableName + " ( "
			+ "FileID " + INT + " NOT NULL, "
			+ "PrevisionRevisionID " + VARCHAR + "(50), "
			+ "RevisionID " + VARCHAR + "(50) NOT NULL, "
			+ "EntityID " + INT + " NOT NULL, "
			+ "Action " + VARCHAR + "(5) NOT NULL, "
			+ "Content " + TEXT + ", "
			+ "CONSTRAINT " + tableName + "_e_fkey "
				+ "FOREIGN KEY (EntityID) REFERENCES " 
				+ getTableName(Tables.TABLE_ENTITIES) + " (EntityID) "
				+ "ON UPDATE CASCADE, "
			+ "CONSTRAINT " + tableName + "_n_fkey "
				+ "FOREIGN KEY (FileID, RevisionID) REFERENCES " 
				+ Tables.TABLE_REVISIONS + " (FileID, RevisionID) "
				+ "ON UPDATE CASCADE "
			+ ");";
		executeUpdate(query);	
		
		// TODO Index required:
		// query = "CREATE INDEX " + tableName + "_idx " + "ON "	+ tableName + "(FileID, NewRevision)";

	}

	
	
	
	/**
	 * 
	 * @param tableName
	 * @throws SQLException
	 */
	public void dropTable(String tableName) throws SQLException {		
		String query = "DROP TABLE " + tableName + ";";
	    executeUpdate(query);
	}

	
	/**
	 * 
	 * @param indexName
	 * @throws SQLException
	 */
	public void dropIndex(String indexName) throws SQLException {		
		String query = "DROP INDEX " + indexName + ";";
	    executeUpdate(query);
	}

	
	public void dropConstraint(String tableName, String constraintName) 
		throws SQLException {
	    
		String query = "ALTER TABLE " + tableName + " "
			+ "DROP CONSTRAINT " + constraintName; // + "RESTRICT";
		executeUpdate(query);
	}
	
	/**
	 * Tests whether a table exists. 
	 * @param tableName
	 * @return
	 */
	public boolean existsTable(String tableName) {
	    // Use "SELECT COUNT(*) FROM " + tableName + ";";
	    // But this should be faster (except for braindead databases):
	    String query = "SELECT 1 FROM " + tableName + " WHERE 1=0;";
	    try {
            stmt.executeQuery(query);
        } catch (SQLException e) {
            return false;
        }
	    return true;
	}
	

	public String getTableName(String tableID) {
	    return Tables.PREFIX_CVS + tableID;
	}


	/**
	 * 
	 * @param entityID
	 * @param entityName
	 * @param fileID
	 * @throws SQLException
	 */
	public void insertEntity(Entity entity) throws SQLException {
	
	    stmtEntities.setInt(1, entity.getEntityId());
	    stmtEntities.setString(2, entity.getId());
	    stmtEntities.setInt(3, entity.getFileId());
	    stmtEntities.execute();	    
	}

	
	/**
	 * 
	 * @param entityID
	 * @param entityName
	 * @param fileID
	 * @throws SQLException
	 */
	public void insertEntity(int entityID, String entityName, int fileID) 
		throws SQLException {
	
	    stmtEntities.setInt(1, entityID);
	    stmtEntities.setString(2, entityName);
	    stmtEntities.setInt(3, fileID);
	    stmtEntities.execute();	    
	}

	
	/**
	 * 
	 * @param entityID
	 * @param entityName
	 * @param fileID
	 * @throws SQLException
	 */
	public void insertChangedEntity(ChangedEntity entity) throws SQLException {
	
	    String content = entity.getContent();
	    content = content!=null ? content.replace((char) 0, ' ') : null;
	    //if (content!=null) System.out.println(content);
	    
	    stmtChangedEntities.setInt(1, entity.getFileId());
	    stmtChangedEntities.setString(2, entity.getPreviousRevisionId());
	    stmtChangedEntities.setString(3, entity.getRevisionId());
	    stmtChangedEntities.setInt(4, entity.getEntityId());
	    stmtChangedEntities.setString(5, entity.getAction());
	    stmtChangedEntities.setString(6, content);
	    stmtChangedEntities.execute();	    
	}

	/**
	 * 
	 * @param entityID
	 * @param entityName
	 * @param fileID
	 * @throws SQLException
	 */
	public void insertChangedEntity
		(int fileID, String previousRevisionID, String revisionID, int entityID, String action) 
		throws SQLException {
	
	    stmtChangedEntities.setInt(1, fileID);
	    stmtChangedEntities.setString(2, previousRevisionID);
	    stmtChangedEntities.setString(3, revisionID);
	    stmtChangedEntities.setInt(4, entityID);
	    stmtChangedEntities.setString(5, action);
	    stmtChangedEntities.setString(6, null);
	    stmtChangedEntities.execute();	    
	}

	
	public void renameTable(String oldName, String newName) 
		throws SQLException {
	    
		String query = "ALTER TABLE " + oldName	+ " RENAME TO " + newName;
		executeUpdate(query);
	}
	
	
	public void setAutoCommit(boolean autoCommit) throws SQLException {
	    connection.setAutoCommit(autoCommit);
	}
	
		
	private PreparedStatement createInsertStatement(String tableName, int numberOfFields) throws SQLException {
	    StringBuffer query = new StringBuffer("INSERT INTO " + tableName + " VALUES (");
	    for (int i = 0; i < numberOfFields; i++) {
            if (i>0) {
                query.append(",?");
            } else {
                query.append("?");
            }
        }
	    query.append(");");
	    PreparedStatement result = connection.prepareStatement(query.toString());
		return result;
	}	

	
	public void executeUpdate(String query) throws SQLException {
	    EntitiesPlugin.debugInfo(query);
		stmt.executeUpdate(query);	    
	}

	public ResultSet executeQuery(String query) throws SQLException {
	    return stmt.executeQuery(query);
	}

}
