/*
 * $Id: GroupJob.java,v 1.7 2004/12/28 21:47:27 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.data.mirror;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.HashSet;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.core.runtime.jobs.Job;
import org.softevo.rose.ccvs.model.TransactionItem;
import org.softevo.rose.data.mirror.db.DatabaseOperations;
import org.softevo.rose.data.mirror.db.Tables;

/**
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class GroupJob extends Job {

    public static final int WORK = 50;

    private DatabaseOperations dbOperations;

    /**
     * @param name
     */
    public GroupJob(DatabaseOperations dbOperations) {
        super("Group revisions to transactions");
        this.dbOperations = dbOperations;
    }

    
    /**
     * @see org.eclipse.core.runtime.jobs.Job#run(org.eclipse.core.runtime.IProgressMonitor)
     */
    protected IStatus run(final IProgressMonitor monitor) {
        try {
            
            monitor.beginTask("Infer transactions", WORK);

            // The table names
            String revisionTable = dbOperations.getTableName(Tables.TABLE_REVISIONS);
            String transactionTable = dbOperations.getTableName(Tables.TABLE_TRANSACTIONS);
            String tmpTable = transactionTable + "_TMP";
            
            String query = null;
            ResultSet rset = null;
    		
		    // Get the maximal TransaktionID
            SubProgressMonitor pm = new SubProgressMonitor(monitor, 5);
            pm.beginTask("Count transactions", IProgressMonitor.UNKNOWN);
			query = "SELECT MAX(TransaktionID) FROM " + transactionTable;
			rset = dbOperations.executeQuery(query);
			rset.next();
			int maxTransaktionID = rset.getInt(1);
			rset.close();
			pm.done();
   
			// Drop foreign key constraint in table Revision and Transaction
			dbOperations.dropConstraint(revisionTable, revisionTable + "_t_fkey");

			// Copy transaction table but drop tmpTable first
		    if (dbOperations.existsTable(tmpTable)) {
		        dbOperations.dropTable(tmpTable);
		    }
			query = "SELECT * INTO " + tmpTable + " FROM " + transactionTable;
			dbOperations.executeUpdate(query);

			// Drop old table
		    if (dbOperations.existsTable(transactionTable)) {
		        dbOperations.dropTable(transactionTable);
		    }
			
		    // Re-create constraints (for efficient join)
		    String[] columnList = { "TransaktionID" };
		    dbOperations.addConstraintUnique(tmpTable + "_unique", tmpTable, columnList);
		    //dbOperations.addConstraintForeignKey(revisionTable + "_t_fkey", 
		    //        revisionTable, columnList, tmpTable, columnList);
		    
		    // Re-create the table Transactions (which is empty)
		    dbOperations.createTableTransactions(transactionTable);	    
			
			// Get all transaktions (using tmpTable and revisionTable)
		    pm = new SubProgressMonitor(monitor, 5);
            pm.beginTask("Get transactions", IProgressMonitor.UNKNOWN);
            query = "SELECT FileID, RevisionID, MessageMD5, Message, Author, CheckinTime "
				+ "FROM " + revisionTable + " r JOIN " + tmpTable + " t "
				+ "ON t.TransaktionID = r.TransaktionID "
				+ "ORDER BY Author, MessageMD5, Message, CheckinTime";
			rset = dbOperations.executeQuery(query);
			pm.done();
			
			// Now we have to drop the constraint in Revisions
			// dbOperations.dropConstraint(revisionTable, revisionTable + "_t_fkey");
			
			final int DT = /* MAX_TIME_DIFFERENCE */ 200 * 1000; // TODO
			int tId = maxTransaktionID;
			long lastCommitDate = 0;

			HashSet hset = new HashSet();
			String author = "__AUTHOR__";
			String messageMD5 = "__MESSAGE__";
			String message = "";
			Timestamp beginTime = null;

			pm = new SubProgressMonitor(monitor, WORK -10);
            pm.beginTask("Group transactions", maxTransaktionID);
			// Iterate over all transactions
			dbOperations.setAutoCommit(false);
			while (rset.next() && !pm.isCanceled()) {

			    // Criterias for grouping transactions:
				// - new author/commiter
				// - new log message
				// - time difference to previous commit > dt seconds
				//     for dt see MAX_TIME_DIFFERENCE
				// - no multiple commits (for a single file)

				int rFileID = rset.getInt("FileID");
				String rRevision = rset.getString("RevisionID");
				String rMessageMD5 = rset.getString("MessageMD5");
				String rAutor = rset.getString("Author");
				long rDate = rset.getTimestamp("CheckinTime").getTime();
				
				// Check if there is a new transaction
				if (!author.equals(rAutor) 
					|| !messageMD5.equals(rMessageMD5)
					|| rDate - lastCommitDate > DT 
					|| hset.contains(new Integer(rFileID))
					|| rset.isLast()) { // isLast inserts the last transaction
	    
					// Ignore first shift (where tId=maxTransaktionID)
					if (tId > maxTransaktionID) {
						// Insert new transaction into new table
					    TransactionItem tx = new TransactionItem(author, message);
					    tx.setBeginTime(beginTime);
					    tx.setEndTime(new Timestamp(lastCommitDate));
						dbOperations.insertTransaktion(tId, tx);
						beginTime = null;
						
						pm.subTask((tId - maxTransaktionID) + " transactions");
					}
					
					if (!rset.isLast()) { tId++; }
					author = rAutor;
					messageMD5 = rMessageMD5;
					message = rset.getString("Message");
					beginTime = new Timestamp(rDate);
					hset = new HashSet();
				}

				// Update: lastCommitDate, hset (=files in current transaction)
				lastCommitDate = rDate;
				hset.add(new Integer(rFileID));

				// Update reference of revision to current transaction
				dbOperations.updateTxInRevision(rFileID, rRevision, tId);

			    pm.worked(1);
			}
			rset.close();
			pm.done();
			dbOperations.setAutoCommit(true);

			// Drop tmp transaktions
			dbOperations.dropTable(tmpTable);

			// Add foreign key constraint to table Revisions
		    dbOperations.addConstraintForeignKey(revisionTable + "_t_fkey", 
		            revisionTable, columnList, transactionTable, columnList);
            
        } catch (SQLException e) {
            MirrorPlugin.logException(e, Status.WARNING);
        } 
        if (monitor.isCanceled()) return Status.CANCEL_STATUS;
        return Status.OK_STATUS;
    }

}
