/*
 * $Id: DBConnectionProperties.java,v 1.12 2005/03/01 12:55:26 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.db;

import java.io.Serializable;
import java.util.Properties;

import org.softevo.rose.db.jdbc.DatabaseProperties;

/** This class contains the properties for
 *  a database connection. <BR>
 *  A properties object is identified by its name.
 */
public class DBConnectionProperties implements Cloneable, Serializable {

    /**
     * 
     */
    private static final long serialVersionUID = 4050201920989639221L;

/** Name constant for internal file system database. <BR>
   */
  public static final String NAME_INTERNALDB = "Internal Database";
  
  /** String key constant for connection name. <BR>
   */
  public static final String KEY_NAME = "CONNECTION_NAME";
  
  /** String key constant for connection url. <BR>
   */
  public static final String KEY_URL = "CONNECTION_URL";
  
  /** String key constant for user name. <BR>
   */
  public static final String KEY_USERNAME = "CONNECTION_USERNAME";

  /** String key constant for password. <BR>
   */
  public static final String KEY_PASSWORD = "CONNECTION_PASSWORD";

  /** A unique identifier for the database
   *  connection. <BR>
   */
  protected String name = null;
  
  /** The url describing the database location. <BR>
   */
  protected String databaseURL = null;

  /** The user name for the database. <BR>
   */
  protected String userName = null;

  /** The password for the database. <BR>
   */
  protected String password = null;
  
  /** The database properties. <BR>
   */
  protected DatabaseProperties databaseProperties = null;

  
  /** Creates a new connection properties object. <BR>
   *  @param name a unique name for the connection
   *  @param databaseURL the database url
   *  @param userName the user name for db access
   *  @param password the password for the account
   *  @param databaseProperties the database properties
   */
  public DBConnectionProperties(String name, String databaseURL, 
                                String userName, String password,
                                DatabaseProperties databaseProperties) {
    this.databaseURL = databaseURL;
    this.userName = userName;
    this.password = password;
    this.name = name;
    this.databaseProperties = databaseProperties;
  }
  
  /** Creates a new connection properties object and
   *  initializes with the values from the object passed. <BR>
   *  @param primaryCopy the primary copy
   */
  public DBConnectionProperties(DBConnectionProperties primaryCopy) {
    this(primaryCopy.name, primaryCopy.databaseURL, 
         primaryCopy.userName, primaryCopy.password,
         (DatabaseProperties) primaryCopy.databaseProperties.clone());
  }
  
  /** Creates a new properties object from a
   *  java properties object. <BR>
   *  The database properties must be set manually
   *  @param properties the properties using the keys in this class
   */
  public DBConnectionProperties(Properties properties) {
    initFromProperties(properties);
  }
  
  /** Inits this object with the properties passed. <BR>
   *  @param properties the properties
   */
  private void initFromProperties(Properties properties) {
    name = properties.getProperty(KEY_NAME);
    databaseURL = properties.getProperty(KEY_URL);
    userName = properties.getProperty(KEY_USERNAME);
    password = decodePassword(properties.getProperty(KEY_PASSWORD));
  }

  /** Sets the properties from this object in
   *  the java properties object. <BR>
   *  @param the java properties object
   */
  public void setProperties(Properties properties) {
    properties.setProperty(KEY_NAME, name);
    properties.setProperty(KEY_URL, databaseURL);
    properties.setProperty(KEY_USERNAME, userName);
    properties.setProperty(KEY_PASSWORD, encodePassword(password));
    properties.setProperty(DatabaseProperties.KEY_NAME, databaseProperties.getName());
  }
  
  /** Gets the database location. <BR>
   */
  public String getDatabaseURL() {
    return databaseURL;
  }

  /** Gets the database user name. <BR>
   */
  public String getUserName() {
    return userName;
  }

  /** Gets the database password. <BR>
   */
  public String getPassword() {
    return password;
  }

  /** Gets the name of the connection. <BR>
   */
  public String getName() {
    return name;
  }

  /** Sets the database url in the object. <BR>
   */
  public void setDatabaseURL(String databaseURL) {
    this.databaseURL = databaseURL;
  }

  /** Sets the user name in the object. <BR>
   */
  public void setUserName(String userName) {
    this.userName = userName;
  }

  /** Sets the password in the object. <BR>
   */
  public void setPassword(String password) {
    this.password = password;
  }

  /** Sets the name in the object. <BR>
   */
  public void setName(String name) {
    this.name = name;
  }

  /** Returns a copy with the same values as this 
   *  object. <BR>
   */
  public Object clone() {
    return new DBConnectionProperties(name, databaseURL, userName, password, databaseProperties);
  }
  
  /** Encodes a password using a simple algorithm. <BR>
   *  @param clearText the clear text password
   *  @return the ciphered password
   */
  private String encodePassword(String clearText) {
    //TODO think of some simple encryption
    return clearText;
  }

  /** Decodes a password using a simple algorithm. <BR>
   *  @param cipherText the ciphered password
   *  @return the clear text password
   */
  private String decodePassword(String cipherText) {
    //TODO think of some simple decryption (prefereably f-1 for encryption)
    return cipherText;
  }

  /** Compares this object to another object. <BR>
   *  @return <code>true</code> if the object is an instance of this class and
   *          matches all properties, <code>false</code> otherwise
   */
  public boolean equals(Object object) {
    DBConnectionProperties properties;
    
    if ((object != null) && (object instanceof DBConnectionProperties)) {
      properties = (DBConnectionProperties) object;
      return properties.databaseURL.equals(databaseURL) &&
             properties.name.equals(name) &&
             properties.password.equals(password) &&
             properties.userName.equals(userName) &&
             properties.databaseProperties.equals(databaseProperties);
    } 
    return false;
  }

  /** Gets the database properties that belong to the connection. <BR>
   */
  public DatabaseProperties getDatabaseProperties() {
    return databaseProperties;
  }
  
  /** Sets the database properties. <BR>
   */
  public void setDatabaseProperties(DatabaseProperties databaseProperties) {
    this.databaseProperties = databaseProperties;
  }

  /** Returns wether or not this
   *  properties object is editable. <BR>
   */
  public boolean isEditable() {
    return true;
  }

}
