/*
 * $Id: DbPlugin.java,v 1.27 2005/01/30 09:38:04 dallmeier Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.db;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.MissingResourceException;
import java.util.NoSuchElementException;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ColorRegistry;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;
import org.softevo.rose.db.jdbc.DatabaseProperties;
import org.softevo.rose.db.jdbc.JdbcPlugin;

/**
 * The main plugin class to be used in the desktop. <BR>
 * This class manages a set of database connection properties
 * and a mapping of project to database driver. All properties
 * objects passed outside are clones and may be modified freely.
 * To alter properties, the appropriate methods must be used.
 * Project specific database data is stored in a file called
 * <code>.rose</code> in the project file. Connection properties
 * are kept in a subdirectory of the DbPlugin directory. 
 */
public class DbPlugin extends AbstractUIPlugin {
  
  /** String constant for project properties file name. <BR>
   */
  public static final String FILE_ROSEPROJECTPROPERTIES = ".rose";
  
  /** String constant for global connection properties file name. <BR>
   */
  public static final String FILE_ROSEPROPERTIES = ".connections";
  
  /** String constant for extension of global properties files. <BR>
   */
  public static final String EXT_ROSEPROPERTIES = ".rose";
  
  /** Symbolic name for white. <BR>
   */
  public static final String COLOR_WHITE = "white";
  
  /** Symbolic name for red. <BR>
   */
  public static final String COLOR_RED = "red";
  
  /** Resource bundle for plugin. <BR>
   */
  private ResourceBundle resourceBundle;
  
  /** The list of database connections configured. <BR>
   */
  private Vector dbConnectionProperties = null;
  
  /** Maps <code>IProject</code> to database provider. <BR>
   */
  private HashMap projectToProviderMap = null;
  
  /** Singleton instance. <BR>
   */
  private static DbPlugin plugin;
  
  /** The color registry. <BR>
   */
  private ColorRegistry colorRegistry = null;
  
  public static final String ID = "org.softevo.rose.db";
  
  /**
   * The constructor.
   */
  public DbPlugin() {
    super();
    plugin = this;
    colorRegistry = createColorRegistry();
    try {
      resourceBundle = ResourceBundle.getBundle("org.softevo.rose.db.DbPluginResources");
    } catch (MissingResourceException x) {
      resourceBundle = null;
    }
  }
  
  /**
   * This method is called upon plug-in activation
   */
  public void start(BundleContext context) throws Exception {
    super.start(context);
    loadDBConnections();
  }
  
  /**
   * This method is called when the plug-in is stopped
   */
  public void stop(BundleContext context) throws Exception {
    super.stop(context);
  }
  
  /**
   * Returns the shared instance.
   */
  public static DbPlugin getDefault() {
    return plugin;
  }
  
  /**
   * Returns the string from the plugin's resource bundle,
   * or 'key' if not found.
   */
  public static String getResourceString(String key) {
    ResourceBundle bundle = DbPlugin.getDefault().getResourceBundle();
    try {
      return (bundle != null) ? bundle.getString(key) : key;
    } catch (MissingResourceException e) {
      return key;
    }
  }
  
  /** Gets the color registry for this plugin. <BR>
   */
  public ColorRegistry getColorRegistry() {
    return colorRegistry;
  }
  
  /**
   * Returns the plugin's resource bundle,
   */
  public ResourceBundle getResourceBundle() {
    return resourceBundle;
  }
  
  /** Returns a copy of the list of connection
   *  properties stored in the plugin. <BR>
   */
  public Vector getDatabaseConnectionProperties() {
    Vector copy;
    
    if (dbConnectionProperties == null) {
      loadDBConnections();
    }
    copy = new Vector();
    for (int counter = 0; counter < dbConnectionProperties.size(); counter++) {
      copy.add(((DBConnectionProperties)dbConnectionProperties.get(counter)).clone());
    }
    return copy;
  }
  
  /** Tests if the plugin contains database connection
   *  properties with the name passed. <BR>
   *  @param name the name of the properties sought
   *  @return <code>true</code> if a corresponding object exists,
   *          <code>false</code> otherwise
   */
  public boolean containsProperties(String name) {
    return getPropertiesIndex(name) != -1;
  }
  
  /** Adds a new connection properties object. <BR>
   *  This method tests if a properties object with
   *  the given name already exists. 
   *  @param name the name for the <b>connection</b> properties
   *  @param properties the <b>database</b> properties
   *  @return a connection properties with name and database properties set
   *  @throws IllegalArgumentException if a properties with the desired name already exists
   */
  public DBConnectionProperties addConnectionProperties(String name, 
                                                        DatabaseProperties properties) throws IllegalArgumentException {
    DBConnectionProperties result;
    
    if (getPropertiesIndex(name) != -1) {
      throw new IllegalArgumentException("Name " + name + " is already in use.");
    } else {
      result = new DBConnectionProperties(name, "", "", "", properties);
      dbConnectionProperties.add(result);
      saveGlobalProperties(dbConnectionProperties.size() - 1);
      return (DBConnectionProperties) result.clone();
    }
  }
  
  /** Changes the corresponding connection properties object. <BR>
   *  @param properties the new values to be set
   *  @throws NoSuchElementException if no properties object with the
   *          given name was found
   */
  public void changeConnectionProperties(DBConnectionProperties properties) throws NoSuchElementException {
    int index;
    
    index = getPropertiesIndex(properties.getName());
    if (index == -1) {
      throw new NoSuchElementException("No properties with name " + properties.getName() +
      " found.");
    }
    saveGlobalProperties(index);
    dbConnectionProperties.set(index, properties.clone());
  }
  
  
  /** Gets all projects that currently use the database connection
   *  with the name passed. <BR>
   *  @param name the name of the database connection
   *  @return an array of the projects
   */
  public IProject[]	getProjectsUsingProperties(String name, String url) {
    Vector projects;
    Iterator iter;
    IProject project;
    DatabaseProvider provider;
    
    projects = new Vector();
    iter = projectToProviderMap.keySet().iterator();
    while (iter.hasNext()) {
      project = (IProject) iter.next();
      provider = (DatabaseProvider) projectToProviderMap.get(project);
      if (provider.getConnectionProperties().getName().equals(name) 
          && provider.getConnectionProperties().getDatabaseURL().equals(url)) {
        projects.add(project);
      }
    }
    return (IProject[]) projects.toArray(new IProject[projects.size()]);  
  }
  
  
  /** Gets all projects that currently use the database connection
   *  with the name passed. <BR>
   *  @param name the name of the database connection
   *  @return an array of the projects
   */
  public IProject[]	getProjectsUsingProperties(String name) {
    Vector projects;
    Iterator iter;
    IProject project;
    DatabaseProvider provider;
    
    projects = new Vector();
    iter = projectToProviderMap.keySet().iterator();
    while (iter.hasNext()) {
      project = (IProject) iter.next();
      provider = (DatabaseProvider) projectToProviderMap.get(project);
      if (provider.getConnectionProperties().getName().equals(name)) {
        projects.add(project);
      }
    }
    return (IProject[]) projects.toArray(new IProject[projects.size()]);  
  }
  
  /**
   * TODO What about the internal database???
   * @param project
   * @return
   */
  public IProject[] getColocatedProjects(IProject project) {
    if (!projectToProviderMap.containsKey(project)) {
      IProject[] singleton = { project };
      return singleton;
    }
    String id = ((DatabaseProvider) projectToProviderMap.get(project)).getConnectionProperties().getName();      
    String url = ((DatabaseProvider) projectToProviderMap.get(project)).getConnectionProperties().getDatabaseURL();
    return getProjectsUsingProperties(id, url);
  }
  
  /** Removes a database connection from the set of connections. <BR>
   *  This method verifies that no project currently uses the
   *  database connection.
   *  @param name the name of the connection
   *  @throws NoSuchElementException if no connection was found
   *  @throws IllegalArgumentException if projects using the connection remain
   */
  public void removeDatabaseConnectionProperties(String name) throws NoSuchElementException, 
  IllegalArgumentException {
    int index;
    IProject[] projects;
    
    index= getPropertiesIndex(name);
    if (index != -1) {
      projects = getProjectsUsingProperties(name);
      if (projects.length > 0) {
        throw new IllegalArgumentException("Properties " + name + " are still in use by " + 
                                           projects.length + " projects.");
      } else {
        dbConnectionProperties.remove(index);
        //save all properties objects
        clearGlobalDirectory(getGlobalDirectory());
        for (int counter = 0; counter < dbConnectionProperties.size(); counter++) {
          saveGlobalProperties(counter);
        }
      }
    } else {
      throw new NoSuchElementException(name);
    }
  }
  
  /** Creates the color registry to be used with
   *  this plugin. <BR>
   */
  private ColorRegistry createColorRegistry() {
    ColorRegistry result;
    
    result = new ColorRegistry();
    result.put(COLOR_WHITE, new RGB(255, 255, 255));
    result.put(COLOR_RED, new RGB(255, 0, 0));
    return result;
  }
  
  /** Gets the index of the database connection identified
   *  by the name passed. <BR>
   *  @param name the name of the connection properties
   *  @return the index or <code>-1</code> if not found
   */
  private int getPropertiesIndex(String name) {
    DBConnectionProperties properties;
    
    for (int counter = 0; counter < dbConnectionProperties.size(); counter++) {
      properties = (DBConnectionProperties) dbConnectionProperties.get(counter);
      if (properties.getName().equals(name)) {
        return counter;
      }
    }
    return -1;
  }
  
  /** Gets the database provider for the project passed. <BR>
   *  If a connection is configured for the project but the
   *  provider was not requested before, a new instance is created.
   *  If no connection is configured for the project passed this
   *  method simply returns <code>null</code>. This method only
   *  returns providers that are connected.
   *  @param project the project 
   *  @return the <code>DatabaseProvider</code> or <code>null</code>
   */
  public DatabaseProvider getProviderForProject(IProject project) {
    DatabaseProvider provider;
    
    provider = (DatabaseProvider) projectToProviderMap.get(project);
    if ((provider != null) && (!provider.isConnected())) {
      try {
        provider.connect();
        return provider;
      } catch (SQLException exception) {
        DbPlugin.logError("An exception occured opening database connection for project " +
                          project.getName() + " to db " + provider.getConnectionProperties().getDatabaseURL(), exception);
        projectToProviderMap.remove(project);
        return null;
      }
    } else if (provider != null) {
      return provider;
    } else {
      return null;
    }
  }
  
  /** This method sets database data for a project. <BR>
   *  If a provider already exists for the specified project,
   *  this method replaces the old driver (after succesfull call to
   *  <code>connect</code>) with the new one. If the new properties
   *  equal the old previously used ones, nothing happens. 
   *  @param project the project to set data for
   *  @param connectionProperties the connection properties
   *  @throws SQLException if the provider could not be openened with the information passed
   */
  public void setProjectDatabaseData(IProject project, 
                                     DBConnectionProperties connectionProperties) throws SQLException {
    DatabaseProvider oldProvider, newProvider;
    
    //TODO verify synchronization
    if (connectionProperties instanceof InternalDBConnectionProperties) {
      connectionProperties = new InternalDBConnectionProperties(project);
    }
    synchronized (projectToProviderMap) {
      oldProvider = (DatabaseProvider) projectToProviderMap.remove(project);
      if ((oldProvider != null) && (oldProvider.getConnectionProperties().equals(connectionProperties))) {
        projectToProviderMap.put(project, oldProvider);
        return;
      } 
      newProvider = new DatabaseProvider(connectionProperties);
      newProvider.connect();
      projectToProviderMap.put(project, newProvider);
      //save properties only when changed
      try {
        saveDatabaseData(project, newProvider);
      } catch (CoreException coreException) {
        DbPlugin.logException(coreException, IStatus.WARNING);
      }
    }
    //TODO use a job for this
    if (oldProvider != null) {
      oldProvider.waitForFreeProvider(); // FIXME This doesnt work. Better: close if there are no users, otherwise do nothing
      try {
        oldProvider.close();
      } catch (SQLException finalizeException) {
        DbPlugin.logException(finalizeException, IStatus.WARNING);
      }
    }
  }
  
  /** Loads the database connections from the persistent
   *  files. <BR>
   *  Loading database connections is done in two steps:
   *  <ol>
   *    <li>Load the serialized vector with all database connections.</li>
   *    <li>Search all projects in the workspace for .rose files. Load the
   *        properties and create the drivers for the project. Properties
   *        from .rose files override properties from the serialized vector</li>
   *  </ol>
   */
  private void loadDBConnections() {
    IProject[] projects;
    IFile rosePropertiesFile;
    Properties properties;
    DatabaseProvider provider;
    DBConnectionProperties connectionProperties;
    DatabaseProperties databaseProperties;
    File directory;
    FileInputStream fileInputStream = null;
    File[] propertiesFiles;
    
    if (dbConnectionProperties == null) {
      dbConnectionProperties = new Vector();
    } else {
      dbConnectionProperties.clear();
    }      
    directory = getGlobalDirectory(); 
    // FIXME Why a SuffixFilter?????
    propertiesFiles = directory.listFiles(new SimpleSuffixFileFilter(EXT_ROSEPROPERTIES));
    Arrays.sort(propertiesFiles);
    properties = new Properties();
    for (int counter = 0; counter < propertiesFiles.length; counter++) {
      try {
        fileInputStream = new FileInputStream(propertiesFiles[counter]);
        properties.clear();
        properties.load(fileInputStream);
        //use factory for correct runtime type
        connectionProperties = DBConnectionPropertiesFactory.loadFromProperties(properties);
        if (getPropertiesIndex(connectionProperties.getName()) == -1) {
          dbConnectionProperties.add(connectionProperties);
        }
      } catch (IOException loadException) {
        DbPlugin.logWarning("An exception occured loading connection properties from file " + 
                            propertiesFiles[counter], loadException);
      } finally {
        if (fileInputStream != null) {
          try {
            fileInputStream.close();
          } catch (IOException finalizeException) {
            //ignore exception
          }
        }
        fileInputStream = null;
      }
    }
    //make sure that internal database connection is present
    if (getPropertiesIndex(InternalDBConnectionProperties.NAME_INTERNALDB) < 0) {
      dbConnectionProperties.insertElementAt(new InternalDBConnectionProperties(), 0);
    }
    if (projectToProviderMap == null) {
      projectToProviderMap = new HashMap();
    } else {
      projectToProviderMap.clear();
    }
    projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
    for (int counter = 0; counter < projects.length; counter++) {
      properties = null;
      rosePropertiesFile = projects[counter].getFile(FILE_ROSEPROJECTPROPERTIES);
      //synchronize with file system
      try {
        rosePropertiesFile.refreshLocal(0, null); // FIXME Is this necessary (see AssistPlugin klz)???? Btw: I changed 1 to 0
      } catch (CoreException exception) {
        DbPlugin.logException(exception, IStatus.WARNING);
      }
      if (rosePropertiesFile.exists()) {
        try {
          properties = new Properties();
          properties.load(rosePropertiesFile.getContents());
          if (properties != null) {
            //we distinguish internal connections from normal ones by name
            if (properties.get(DBConnectionProperties.KEY_NAME).equals(InternalDBConnectionProperties.NAME_INTERNALDB)) {
              connectionProperties = new InternalDBConnectionProperties(projects[counter]);
              provider = new DatabaseProvider(connectionProperties);
              projectToProviderMap.put(projects[counter], provider);
            } else {
              connectionProperties = DBConnectionPropertiesFactory.loadFromProperties(properties);
              databaseProperties = JdbcPlugin.getDefault().getProperties(properties.getProperty(DatabaseProperties.KEY_NAME));
              if (databaseProperties == null) {
                DbPlugin.logWarning("No jdbc driver found for name " + properties.getProperty(DatabaseProperties.KEY_NAME) +
                                    " in project " + projects[counter].getName(), null);
              } else {
                connectionProperties.setDatabaseProperties(databaseProperties);
                provider = new DatabaseProvider(connectionProperties);
                if (!dbConnectionProperties.contains(connectionProperties)) {
                  dbConnectionProperties.add(connectionProperties);
                } else {
                  dbConnectionProperties.set(getPropertiesIndex(connectionProperties.getName()),
                                             connectionProperties);
                }
                projectToProviderMap.put(projects[counter], provider);
              }
            }
          }
        } catch (Exception loadException ) {
          properties = null;
          DbPlugin.logWarning("An exception occured loading rose properties file for " +
                              "project " + projects[counter].getName(), loadException);
        }
      }	
    }
    for (Iterator iter = projectToProviderMap.keySet().iterator(); iter.hasNext();) {
      IProject project = (IProject) iter.next();
      DatabaseProvider prov = (DatabaseProvider) projectToProviderMap.get(project);
      DbPlugin.debugInfo(project + "->" + prov.getConnectionProperties().getDatabaseURL());
    }
  }
  
  /** Saves database data for the specified project. <BR>
   *  @param project the project to save data for
   *  @param provider the database provider
   *  @throws CoreException if an exception occurs in the workspace
   */
  private void saveDatabaseData(IProject project, 
                                DatabaseProvider provider) throws CoreException {
    IFile rosePropertiesFile;
    ByteArrayOutputStream byteBuffer;
    Properties properties;
    
    rosePropertiesFile = project.getFile(FILE_ROSEPROJECTPROPERTIES);
    rosePropertiesFile.refreshLocal(1, null);
    if (!rosePropertiesFile.exists()) {
      rosePropertiesFile.create(null, true, null);
    }
    properties = DBConnectionPropertiesFactory.createProperties(provider.getConnectionProperties());
    byteBuffer = new ByteArrayOutputStream();
    //we use this method since it is save to write to
    //a byte buffer in memory
    properties.save(byteBuffer, "");
    rosePropertiesFile.setContents(new ByteArrayInputStream(byteBuffer.toByteArray()), true, false, null);
  }
  
  /** Gets a reference to the directory that stores the global
   *  database connection properties. <BR>
   */
  private File getGlobalDirectory() {
    File file;
    
    file = new File(getStateLocation().toFile().getAbsolutePath() +
                    File.separator + FILE_ROSEPROPERTIES );
    if (!file.exists()) {
      file.mkdirs();
    }
    return file;
  }
  
  /** Removes all properties files from the
   *  global directory. <BR>
   *  @param directory the global directory for connection properties
   */
  private void clearGlobalDirectory(File directory) {
    File[] children;
    
    children = directory.listFiles(new SimpleSuffixFileFilter(EXT_ROSEPROPERTIES));
    for (int counter = 0; counter < children.length; counter++) {
      children[counter].delete();
    }
  }
  
  /** Closes all active database providers. <BR>
   */
  private void closeProviders() {
    Iterator iter;
    DatabaseProvider provider;
    IProject project;
    
    synchronized(projectToProviderMap) {
      iter = projectToProviderMap.keySet().iterator();
      while (iter.hasNext()) {
        project = (IProject) iter.next();
        provider = (DatabaseProvider) projectToProviderMap.remove(project);
        try {
          if (provider.isConnected()) {
            provider.close();
          }
        } catch (SQLException finalizeException) {
          //ignore exception
        }
      }
    }
  }

  /** Saves the current state of the properties object
   *  indicated into the global state directory. <BR>
   *  @param index the index in the properties list
   */
  private void saveGlobalProperties(int index) {
    File directory;
    FileOutputStream fileOutputStream = null;
    String fileName;
    Properties savedProperties;
    DBConnectionProperties properties;

    directory = getGlobalDirectory();
    fileName = directory.getAbsolutePath() + File.separatorChar +
    "connection_" + index + EXT_ROSEPROPERTIES;
    properties = (DBConnectionProperties) dbConnectionProperties.get(index);
    savedProperties = DBConnectionPropertiesFactory.createProperties(properties);
    try {
      fileOutputStream = new FileOutputStream(fileName);
      savedProperties.store(fileOutputStream, "");
    } catch (IOException writeError) {
      System.out.println("An exception occured saving connection properties" + properties.getName());
    } finally {
      if (fileOutputStream != null) {
        try {
          fileOutputStream.close();
        } catch (IOException finalizeException) {
          //ignore exception
        }
        fileOutputStream = null;
      }
    }
  }

  protected void initializeImageRegistry(ImageRegistry registry) {
    super.initializeImageRegistry(registry);
    put(registry, "newlocation", "icons/newlocation_rose_wizban.gif"); //$NON-NLS-1$//$NON-NLS-2$
  }
  
  private void put(ImageRegistry registry, String key, String fileName) {
    registry.put(key, ImageDescriptor.createFromURL(Platform.find(getBundle(), new Path(fileName))));	
  }
  
  public static void debugInfo(String message) {
    if (plugin.isDebugging())
      System.out.println(message);
  }
  
  public static void logInfo(String message) {
    if (plugin.isDebugging())
      plugin.getLog().log(new Status(IStatus.INFO, ID, 0, message, null));
  }
  public static void logWarning(String message, Throwable e) {
    plugin.getLog().log(new Status(IStatus.WARNING, ID, 0, message, e));
  }
  
  public static void logError(String message, Throwable e) {
    plugin.getLog().log(new Status(IStatus.ERROR, ID, 0, message, e));
  }
  
  public static void logException(Throwable e, int severity) {
    plugin.getLog().log(new Status(severity, ID, 0, e.getMessage(), e));
  } 
  
  class SimpleSuffixFileFilter implements FileFilter {
    
    /** The filename suffixes to search for */
    private String suffix;
    
    /**
     * Constructs a new Suffix file filter for a single extension.
     * 
     * @param suffix  the suffix to allow, must not be null
     * @throws IllegalArgumentException if the suffix is null
     */
    public SimpleSuffixFileFilter(String suffix) {
      if (suffix == null) {
        throw new IllegalArgumentException("The suffix must not be null");
      }
      this.suffix = suffix;
    }
    
    /**
     * Checks to see if the filename ends with the suffix.
     * 
     * @param file  the File to check
     * @return true if the filename ends with one of our suffixes
     */
    public boolean accept(File file) {
      String name = file.getName();
      return name.endsWith(this.suffix);
    }    
  }
  
}
