/*
 * $Id: DatabaseProperties.java,v 1.9 2005/03/01 12:55:25 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.db.jdbc;

import java.io.Serializable;
import java.util.Properties;

/** This class groups information about the jdbc driver
 *  and type mapping needed to connect to a specific database
 *  type. <BR>
 */
public class DatabaseProperties implements Cloneable, Serializable {

    private static final long serialVersionUID = 3258416140202946870L;

/** String constant for properties key for database name. <BR>
   */
  public static final String KEY_NAME = "DATABASE_NAME";

  /** SQL type constant for type int. <BR>
   */
  public static final String TP_INT = "INT";

  /** SQL type constant for type varchar. <BR>
   */
  public static final String TP_VARCHAR = "VARCHAR";
  
  /** SQL type constant for type text. <BR>
   */
  public static final String TP_TEXT = "TEXT";
  
  /** SQL type constant for type timestamp. <BR>
   */
  public static final String TP_TIMESTAMP = "TIMESTAMP";

  /** The name of the jdbc driver class. <BR>
   */
  private String jdbcDriverClassName = null; 

  /** An identifier for the database properties. <BR>
   */
  private String name = null;
  
  /** The map that stores sql type mapping information. <BR>
   */
  private Properties typeMap = null;

  /** Creates a new database properties instance. <BR>
   *  @param name a unique identifier
   *  @param jdbcDriverClassName the jdbc driver class
   */
  public DatabaseProperties(String name, String jdbcDriverClassName) {
    this.name = name;
    this.jdbcDriverClassName = jdbcDriverClassName;
    this.typeMap = createDefaultTypeMap();
  }
  
  /** Sets <b>only</b> the name in the java
   *  properties object. <BR>
   *  @param properties the properties object
   */
  public void setProperties(Properties properties) {
    properties.setProperty(KEY_NAME, name);
  }
  
  /** Sets the jdbc driver class name. <BR>
   *  @param jdbcDriverClassName the jdbc driver
   */
  void setJdbcDriverClassName(String jdbcDriverClassName) {
    this.jdbcDriverClassName = jdbcDriverClassName;
  }
  
  /** Gets the jdbc driver class name. <BR>
   */
  public String getJdbcDriverClassName() {
    return jdbcDriverClassName;
  }

  /** Gets the properties identifier. <BR>
   */
  public String getName() {
    return name;
  }
  
  /** Sets the name for the properties entry. <BR>
   */
  protected void setName(String name) {
    this.name = name;
  }

  /** Sets a type mapping. <BR>
   *  @param sqlType the type to be mapped. <BR>
   *  @param mapping the mapping for this db
   */
  public void setTypeMapping(String sqlType, String mapping) {
    typeMap.setProperty(sqlType, mapping);
  }
  
  /** Gets the mapping for the sql type passed. <BR>
   *  @param sqlType the sql type
   *  @return the mapping for this db or <code>null</code>
   */
  public String getTypeMapping(String sqlType) {
    return typeMap.getProperty(sqlType);
  }

  /** Creates a default type mapping that should
   *  match most database requirements. <BR>
   *  @return the default type map
   */
  protected Properties createDefaultTypeMap() {
    Properties result;
    
    result = new Properties();
    result.setProperty(TP_INT, TP_INT);
    result.setProperty(TP_TEXT, TP_TEXT);
    result.setProperty(TP_TIMESTAMP, TP_TIMESTAMP);
    result.setProperty(TP_VARCHAR, TP_VARCHAR);
    return result;
  }
  
  /** Clones the properties object. <BR>
   */
  public Object clone() {
    DatabaseProperties result;
    
    result = new DatabaseProperties(name, jdbcDriverClassName);
    result.typeMap = (Properties) typeMap.clone();
    return result;
  }

  /** Two properties are equal if all elements match. <BR>
   */
  public boolean equals(Object object) {
    DatabaseProperties properties;

    if ((object != null) && (object instanceof DatabaseProperties)) {
      properties = (DatabaseProperties) object;
      return properties.jdbcDriverClassName.equals(jdbcDriverClassName) &&
          	properties.name.equals(name);

    } 
    return false;
  }
}
